;*******************************************************************************
; File: fit.pro
;
; Description:
; ------------
; Container for procedures of the AMOEBA software related to fitting 
; of data.
;
; Block directory:
; ----------------
;
; Block 1: init_marquardtfit,init_modelfit
;	   init_fitinterferometry,init_fitastrometry
;
; Block 2: fitellipse1,fitorbit1,marquardt1,levenberg1
;
; Block 3: fitmodel,modelfitparmcopy,
;	   fitellipse,fitorbit
;
; Block 4: marquardt,
;	   fixell,ellfuncs,
;	   orbfuncs,
;	   modfuncs,
;	   funct_gauss,mask2pix,maskfuncs
;
; Block 5: setorbit,setmodel
;
; Block 6: fitnights
;
;************************************************************************Block 1
function init_marquardtfit
;
; Initialize global Marquardt-Levenberg fit options.
;
common MarquardtFit,fit_options
;
fit_options=alloc_fit_options()
;
return,0
;
end
;-------------------------------------------------------------------------------
function init_modelfit
;
; Initialize parameters for modelfit. Clear only the parameters listed 
; for the global modelfit.
; Return non-zero value to indicate error occured.
;
common ModelFit,parameters,ds_options
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
;
if n_elements(gen_model) eq 0 then begin
	print,'***Error(INIT_MODELFIT): no model!'
	return,-1
endif
;
if n_elements(parameters) eq 0 then begin
	num_system=1
	component=alloc_component()
	parameters=replicate(component,num_star()+num_binary()+num_system)
endif else begin
	for i=0,n_elements(parameters.component)-1 do begin
		erase=0
		if strlen(parameters(i).component) eq 1 then erase=1
		index=where(parameters(i).names ne 'Rho' $
			and parameters(i).names ne 'Theta' $
			and strlen(parameters(i).names) ne 0,erase)
		if erase gt 0 then begin
			parameters(i).component=''
			parameters(i).names=''
		endif
	endfor
endelse
;
return,0
;
end
;-------------------------------------------------------------------------------
function init_fitinterferometry
;
; Initialize parameters for modelfit. Clear only the parameters listed 
; for the interferometric modelfit.
; Return non-zero value to indicate error occured.
;
common ModelFit,parameters,ds_options
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
;
if n_elements(gen_model) eq 0 then begin
	print,'***Error(INIT_FITINTERFEROMETRY): no model!'
	return,-1
endif
;
if n_elements(parameters) eq 0 then begin
	num_system=1
	component=alloc_component()
	parameters=replicate(component,num_star()+num_binary()+num_system)
endif else begin
	for i=0,n_elements(parameters.component)-1 do begin
		index=where(parameters(i).names eq 'Rho' $
			 or parameters(i).names eq 'Theta',count)
		if count gt 0 then parameters(i).component=''
	endfor
endelse	
;
nights=''
;
return,0
;
end
;-------------------------------------------------------------------------------
function init_fitastrometry
;
; Initialize parameters for modelfit. Clear only the parameters listed 
; for the astrometric modelfit.
; Return non-zero value to indicate error occured.
;
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
;
; Ellipse
ellipse_options=alloc_ellipse_options()
if n_elements(e_parms) eq 0 then e_parms=dblarr(5)
;
; Orbit
orbit_options=alloc_orbit_options()
if n_elements(o_parms) eq 0 then o_parms=dblarr(8)
;
return,0
;
end
;************************************************************************Block 2
pro fitellipse1
;
; Fit apparent ellipse to position data. Use external library functions.
;
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
; Check data
if n_elements(positions) eq 0 then begin
	print,'***Error(FITELLIPSE): no data!'
	return
endif
;
; Check ellipse parameters
if n_elements(e_parms) eq 0 then begin
	print,'***Error(FITELLIPSE): no ellipse parms!'
	return
endif
;
; Set data. Shift data to array indices 1:n.
index=where(positions.rho gt 0,ndata)
if ndata gt 0 then begin
	p=[0.d0,positions(index).theta]
	r=[0.d0,positions(index).rho]
	sig=dblarr(ndata+1)+1
endif else begin
	print,'***Error(FITELLIPSE): no valid data!'
	return
endelse
;
; Set parameters to fit
ma=n_elements(e_parms)
ia=lonarr(ma+1)
if ellipse_options.all then ia(*)=1 else begin
	if ellipse_options.c then ia(1:2)=1
	if ellipse_options.a then ia(3)=1
	if ellipse_options.b then ia(4)=1
	if ellipse_options.p then ia(5)=1
endelse
index=where(ia ne 0,mfit)
if mfit eq 0 then begin
	print,'***Error(FITELLIPSE): no fit parameters selected!'
	return
endif
a=[0.d0,e_parms]
;
; Store ellipse data
ex=r*sin(p)
ey=r*cos(p)
status=linknload(!external_lib,'store_ell',ex,ey,ndata)
;
; Call marquardt
marquardt1,'ellfuncs',p,r,sig,ndata,a,ia,ma,chisq,covar
;
; Set new parameters
e_parms=a(1:ma)
RAD=180/pi_circle
print,'__________________________________'
print,'Center x = ',e_parms(0)
print,'Center y = ',e_parms(1)
print,'Semi-major axis = ',e_parms(2)
print,'Semi-minor axis = ',e_parms(3)
print,'Position  angle = ',e_parms(4)*RAD
print,'_______________***________________'
;
; Free memory allocated in store_ell
status=linknload(!external_lib,'free_ell',ndata)
;
end
;-------------------------------------------------------------------------------
pro fitorbit1
;
; Fit orbital elements to position data. Use external library functions.
;
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
; Check data
if n_elements(positions) eq 0 then begin
	print,'***Error(FITORBIT): no data!'
	return
endif
;
; Check orbit parameters
if n_elements(o_parms) eq 0 then begin
	print,'***Error(FITORBIT): no orbit parms!'
	return
endif
;
; Check whether orbit parameters have been initialized
if total(o_parms) eq 0 then begin
	print,'***Error(FITORBIT): bad starting model!'
	return
endif
;
; Set data. Shift data to array indices 1:n.
index=where(positions.rho gt 0,ndata)
if ndata gt 0 then begin
	jd=[0.d0,positions(index).jd]
	p=[0.d0,positions(index).theta]
	r=[0.d0,positions(index).rho]
	sig=[1.d0,positions(index).emajor]
endif else begin
	print,'***Error(FITORBIT): no valid data!'
	return
endelse
;
; Set parameters to fit
; RAD=180/3.141592653d0
; o_parms=dblarr(8)
; o_parms(0)=50.0
; o_parms(1)=0.20
; o_parms(2)=127.0/RAD
; o_parms(3)=80.0/RAD
; o_parms(4)=20.0/RAD
; o_parms(5)=103.0
; o_parms(6)=7518.0
;
; Note: o_parms(8) is the orbital sense (not fit).
ma=n_elements(o_parms)-1
ia=lonarr(ma+1)
if orbit_options.all then ia(*)=1 else begin
	if orbit_options.a then ia(1)=1
	if orbit_options.e then ia(2)=1
	if orbit_options.i then ia(3)=1
	if orbit_options.w then ia(4)=1
	if orbit_options.n then ia(5)=1
	if orbit_options.p then ia(6)=1
	if orbit_options.t then ia(7)=1
endelse
index=where(ia ne 0,mfit)
if mfit eq 0 then begin
	print,'***Error(FITORBIT): no fit parameters selected!'
	return
endif
a=[0.d0,o_parms(0:6)]
;
; Store ellipse data
ex=r*sin(p)
ey=r*cos(p)
status=linknload(!external_lib,'store_ell',ex,ey,ndata)
;
; Compute chisq
; xy=true2app(jd,o_parms)
; xy(0,*)=0
; chisq=total(((ex-xy(*,0))/sig)^2+((ey-xy(*,1))/sig)^2)
;
; Set r to zero since orbfuncs does the differences
r=dblarr(ndata+1)
;
; Call marquardt1
marquardt1,'orbfuncs',jd,r,sig,ndata,a,ia,ma,chisq,covar
;
; Set new paramters
o_parms(0:6)=a(1:ma)
RAD=180/pi_circle
print,'__________________________________'
print,'Semi-major axis = ',o_parms(0)
print,'Eccentricity =    ',o_parms(1)
print,'Inclination =     ',o_parms(2)*RAD
print,'Periastron =      ',o_parms(3)*RAD
print,'Ascending node =  ',o_parms(4)*RAD
print,'Period =          ',o_parms(5)
print,'Epoch =           ',o_parms(6)
print,'_______________***________________'
;
; Free memory allocated in store_ell
status=linknload(!external_lib,'free_ell',ndata)
;
end
;-------------------------------------------------------------------------------
pro marquardt1,func,x,y,sig,ndata,a,ia,ma,chisq,covar,alamda
;
; Iterative procedure to do non-linear least-squares fit.
;
common MarquardtFit,fit_options
;
if n_elements(fit_options) eq 0 then if init_marquardtfit() ne 0 then return
;
covar=dblarr(ma,ma)
alpha=dblarr(ma,ma)
;
chifr=fit_options.chifr
tol=fit_options.tolerance
alamda=-fit_options.alamda
;
status=linknload(!external_lib,'marquardt_init',ma)
;
if fit_options.s then mode=10L else mode=0L
if fit_options.e then mode=mode+1
;
chisq=0.d0
status=linknload(!external_lib,'marquardt', $
		x,y,sig,ndata,a,ia,ma,covar,alpha,chisq,func,alamda,tol,mode)
print,''
print,'Initial chisq=',chisq
;
i=1
repeat begin
	chisqo=chisq
	status=linknload(!external_lib,'marquardt', $
		x,y,sig,ndata,a,ia,ma,covar,alpha,chisq,func,alamda,tol,mode)
	i=i+1
endrep until (1-chisq/chisqo lt chifr and chisq lt chisqo) $
	or alamda gt 10 $
	or i gt 100 $
	or status ne 0 $
	or fit_options.o
print,'Final   chisq=',chisq
print,'Iter=',i,', Alamda=',alamda,', Status=',status, $
	format='(a,i2,a,f8.2,a,i1)'
;
alamda=0.d0
status=linknload(!external_lib,'marquardt', $
		x,y,sig,ndata,a,ia,ma,covar,alpha,chisq,func,alamda,tol,mode)
;
status=linknload(!external_lib,'marquardt_free',ma)
;
end
;-------------------------------------------------------------------------------
pro levenberg1,func,x,y,a
;
if n_params() eq 0 then begin
	print,'Procedure LEVENBERG:'
	print,'Please supply: func, x, y, a!'
	return
endif
;
ndata=n_elements(x)
ma=n_elements(a)
;
xx=double([0,x])
yy=double([0,y])
sigma=dblarr(ndata+1)+1
aa=double([0,a])
;
ia=lonarr(ma+1)+1
;
chisq=1d6
init_fit
;
marquardt1,func,xx,yy,sigma,ndata,aa,ia,ma,chisq,covar,alamda
;
end
;************************************************************************Block 3
pro fitmodel,chisq,ndata
;
; Global model fit.
; Fit model of hierarchical stellar system to combined data sets.
;
common ModelFit,parameters,ds_options
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
common SysConfig,SystemId,Date,MetroConfig,GenConfig,GeoParms,GenInfo,GeoInfo
;
; Check data
if ds_options.i and n_elements(scans) eq 0 then begin
	print,'***Error(FITMODEL): no interferometric data!'
	return
endif
if ds_options.a and n_elements(positions) eq 0 then begin
	print,'***Error(FITMODEL): no astrometric data!'
	return
endif
;
; Check parameters
icheck=0
if n_elements(parameters) eq 0 then icheck=1 else $
if fix(total(strlen(parameters.names))) eq 0 then icheck=1
if icheck then begin
	print,'***Error(FITMODEL): no modelfit parameters defined!'
	return
endif
;
; Initialize parameter array a with hierarchical model
modelfitparmcopy,-1,a
ma=n_elements(a)
ia=lonarr(ma)+1
mfit=ma
if mfit eq 0 then begin
	print,'***Error(FITMODEL): no fit parameters selected!'
	return
endif
;
; Set data.
calcmodel
marquardtdata,y,ysig,ymod
ndata=n_elements(y)
if ndata eq 1 and ysig(0) eq 0 then ndata=0
if ndata lt ma then begin
	print,'***Error(FITMODEL): not enough data!'
	return
endif
x=dindgen(ndata)
;
; Call marquardt
marquardt,'modfuncs',x,y,ysig,ndata,a,ia,ma,chisq,covar
if ndata-mfit ne 0 then chisq=chisq/(ndata-mfit)
;
print,'Normalized covariance matrix:'
for i=0,n_elements(parameters)-1 do begin
	j=where(strlen(parameters(i).names) ne 0,n)
	for j=0,n-1 do print,parameters(i).names(j),format='(1x,a5,$)'
endfor
print,''
j=indgen(ma)
for i=0,ma-1 do print,covar(j,i)/sqrt(covar(j,j)*covar(i,i)), $
	format='(100(1x,f5.2))'
;
; Store new parameters into hierarchical model
modelfitparmcopy,1,a
;
; Store diagonal elements of covariance matrix
; ...save the model first, and replace with model errors
gen_model_bck=gen_model
gen_model=gen_error
star_model_bck=star_model
star_model=star_error
binary_model_bck=binary_model
binary_model=binary_error
; ...extract diagonal elements of covariance matrix
index=indgen(n_elements(a))
e=sqrt(covar(index,index))
modelfitparmcopy,1,e
; ...copy model errors, replace model
gen_error=gen_model
gen_model=gen_model_bck
star_error=star_model
star_model=star_model_bck
binary_error=binary_model
binary_model=binary_model_bck
;
print,'FitModel complete.'
;
end
;-------------------------------------------------------------------------------
pro modelfitparmcopy,direction,a,da
;
; Copies parameters in array a from (direction=-1) and to (direction=1) 
; hierarchical stellar model. Also initializes array da which contains
; increments for the parameters to be used in the differential corrections.
; Procedure determines for each parameter listing the component, whether
; it is a binary component or something else, i.e. either star or general
; model. Since the parameter names are currently unique, this is not really
; necessary for the division of the case statements into two sections.
;
; Some parameter safeguards are implemented (e.g. e<1)
;
common ModelFit,parameters,ds_options
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
;
maxparm=1000
if direction eq -1 then a=dblarr(maxparm)
da=a
;
n=0	; ID in flat parameter array
;
FOR j=0,n_elements(parameters)-1 DO BEGIN
;
if strlen(parameters(j).component) gt 0 then begin
binary=strpos(parameters(j).component,'-')
if binary gt 0 then begin
	ic=where(binary_model.component eq parameters(j).component) & ic=ic(0)
endif else begin
	ic=where(star_model.component eq parameters(j).component) & ic=ic(0)
endelse
index=where(strlen(parameters(j).names) gt 0,np)
for i=0,np-1 do begin
	parameter=parameters(j).names(i)
	if binary gt 0 then begin
	case parameter of
		'Semimajoraxis': begin
			if direction eq 1 then $
				binary_model(ic).semimajoraxis=abs(a(n)) else $
				a(n)=binary_model(ic).semimajoraxis
			da(n)=0.01	; mas
			end
		'Inclination': begin
			if direction eq 1 then $
				binary_model(ic).inclination=a(n) mod 360 else $
				a(n)=binary_model(ic).inclination
			da(n)=0.01	; deg
			end
		'Ascendingnode': begin
			if direction eq 1 then $
				binary_model(ic).ascendingnode=a(n) mod 360 else $
				a(n)=binary_model(ic).ascendingnode
			da(n)=0.01/(sin(binary_model(ic).inclination)+0.001)
			end
		'Eccentricity': begin
			if direction eq 1 then $
				binary_model(ic).eccentricity=a(n) < 0.99 else $
				a(n)=binary_model(ic).eccentricity
			da(n)=0.001
			end
		'Periastron': begin
			if direction eq 1 then $
				binary_model(ic).periastron=a(n) else $
				a(n)=binary_model(ic).periastron
			da(n)=0.01/(binary_model(ic).eccentricity^2+0.001)
			end
		'Apsidalmotion': begin
			if direction eq 1 then $
				binary_model(ic).apsidalmotion=a(n) else $
				a(n)=binary_model(ic).apsidalmotion
			da(n)=0.01/(binary_model(ic).eccentricity^2+0.001)
			end
		'Epoch': begin
			if direction eq 1 then $
				binary_model(ic).epoch=a(n) else $
				a(n)=binary_model(ic).epoch
			da(n)=binary_model(ic).period/1000
			end
		'Period': begin
			if direction eq 1 then $
				binary_model(ic).period=abs(a(n)) else $
				a(n)=binary_model(ic).period
			da(n)=binary_model(ic).period/10000
			end
		'Rho': begin
			if direction eq 1 then $
				binary_model(ic).rho=abs(a(n)) else $
				a(n)=binary_model(ic).rho
			da(n)=0.01
			end
		'Theta': begin
			if direction eq 1 then $
				binary_model(ic).theta=a(n) mod 360 else $
				a(n)=binary_model(ic).theta
			da(n)=0.001
			end
	endcase
	endif else begin
	pos=strpos(parameters(j).names(i),'Magnitude')
	if pos ge 0 then begin
		wave=long(strmid(parameter,10,4))
		ch=where(fix(gen_model.wavelengths*1000) eq wave)
		parameter=strmid(parameter,0,9)
	endif
	case parameter of
		'RV':	begin
			if direction eq 1 then $
				gen_model.rv=a(n) else $
				a(n)=gen_model.rv
			da(n)=0.1
			end
		'Mass':	begin
			if direction eq 1 then $
				star_model(ic).mass=a(n) else $
				a(n)=star_model(ic).mass
			da(n)=0.1
			end
		'Magnitude': begin
			if direction eq 1 then $
				star_model(ic).magnitudes(ch)=a(n) else $
				a(n)=star_model(ic).magnitudes(ch)
			da(n)=0.01
			end
		'Diameter': begin
			if direction eq 1 then $
				star_model(ic).diameter=abs(a(n)) else $
				a(n)=abs(star_model(ic).diameter)
			da(n)=0.02
			end
		'Omega': begin
			if direction eq 1 then $
				star_model(ic).omegabr=a(n) < 0.99 else $
				a(n)=star_model(ic).omegabr
			da(n)=0.02
			end
		'Ratio': begin
			if direction eq 1 then $
				star_model(ic).ratio=abs(a(n)) else $
				a(n)=abs(star_model(ic).ratio)
			if a(n) gt 1 then begin
				a(n)=1/a(n)
				star_model(ic).ratio=a(n)
				star_model(ic).pa=(star_model(ic).pa+90) mod 360
			endif
			da(n)=0.05
			end
		'PA': begin
			if direction eq 1 then $
				star_model(ic).pa=a(n) mod 360 else $
				a(n)=star_model(ic).pa mod 360
			da(n)=2.0
			end
		'Tilt': begin
			if direction eq 1 then $
				star_model(ic).tilt=a(n) else $
				a(n)=star_model(ic).tilt
			da(n)=2.0
			end
		'Teff': begin
			if direction eq 1 then $
				star_model(ic).teff=a(n) else $
				a(n)=star_model(ic).teff
			da(n)=100
			end
	endcase
	endelse
	n=n+1
endfor
endif
;
ENDFOR
;
if direction eq -1 then begin
	if n eq 0 then a=0 else a=a(0:n-1)
endif
;
end
;-------------------------------------------------------------------------------
pro fitellipse
;
; Fit apparent ellipse to position data.
;
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
common FitEllipse,ex,ey
;
; Check data
if n_elements(positions) eq 0 then begin
	print,'***Error(FITELLIPSE): no data!'
	return
endif
;
; Check ellipse parameters
if n_elements(e_parms) eq 0 then begin
	print,'***Error(FITELLIPSE): no ellipse parms!'
	return
endif
;
; Set data.
index=where(positions.rho gt 0,ndata)
if ndata gt 0 then begin
	p=positions(index).theta
	r=positions(index).rho
	sig=dblarr(ndata)+1
endif else begin
	print,'***Error(FITELLIPSE): no valid data!'
	return
endelse
;
; Set parameters to fit
ma=n_elements(e_parms)
ia=lonarr(ma)
if ellipse_options.all then ia(*)=1 else begin
	if ellipse_options.c then ia(0:1)=1
	if ellipse_options.a then ia(2)=1
	if ellipse_options.b then ia(3)=1
	if ellipse_options.p then ia(4)=1
endelse
index=where(ia ne 0,mfit)
if mfit eq 0 then begin
	print,'***Error(FITELLIPSE): no fit parameters selected!'
	return
endif
a=e_parms
;
; Store ellipse data
ex=r*sin(p)
ey=r*cos(p)
;
; Call marquardt
marquardt,'ellfuncs',p,r,sig,ndata,a,ia,ma,chisq,covar
;
; Set new parameters
e_parms=a
RAD=180/pi_circle
print,'__________________________________'
print,'Center x = ',e_parms(0)
print,'Center y = ',e_parms(1)
print,'Semi-major axis = ',e_parms(2)
print,'Semi-minor axis = ',e_parms(3)
print,'Position  angle = ',e_parms(4)*RAD
print,'_______________***________________'
;
end
;-------------------------------------------------------------------------------
pro fitorbit
;
; Fit orbital elements to position data.
;
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
common FitEllipse,ex,ey
;
; Check data
if n_elements(positions) eq 0 then begin
	print,'***Error(FITORBIT): no data!'
	return
endif
;
; Check orbit parameters
if n_elements(o_parms) eq 0 then begin
	print,'***Error(FITORBIT): no orbit parms!'
	return
endif
;
; Check whether orbit parameters have been initialized
if total(o_parms) eq 0 then begin
	print,'***Error(FITORBIT): bad starting model!'
	return
endif
;
; Set data.
index=where(positions.rho gt 0,ndata)
if ndata gt 0 then begin
	jd=positions(index).jd
	p=positions(index).theta
	r=positions(index).rho
	sig=positions(index).emajor
endif else begin
	print,'***Error(FITORBIT): no valid data!'
	return
endelse
;
; Set parameters to fit
; RAD=180/3.141592653d0
; o_parms=dblarr(8)
; o_parms(0)=50.0
; o_parms(1)=0.20
; o_parms(2)=127.0/RAD
; o_parms(3)=80.0/RAD
; o_parms(4)=20.0/RAD
; o_parms(5)=103.0
; o_parms(6)=7518.0
;
; Note: o_parms(8) is the orbital sense (not fit).
ma=n_elements(o_parms)-1
ia=lonarr(ma)
if orbit_options.all then ia(*)=1 else begin
	if orbit_options.a then ia(0)=1
	if orbit_options.e then ia(1)=1
	if orbit_options.i then ia(2)=1
	if orbit_options.w then ia(3)=1
	if orbit_options.n then ia(4)=1
	if orbit_options.p then ia(5)=1
	if orbit_options.t then ia(6)=1
endelse
index=where(ia ne 0,mfit)
if mfit eq 0 then begin
	print,'***Error(FITORBIT): no fit parameters selected!'
	return
endif
a=o_parms(0:6)
;
; Store ellipse data
ex=r*sin(p)
ey=r*cos(p)
;
; Compute chisq
; xy=true2app(jd,o_parms)
; xy(0,*)=0
; chisq=total(((ex-xy(*,0))/sig)^2+((ey-xy(*,1))/sig)^2)
;
; Set r to zero since orbfuncs does the differences
r=dblarr(ndata)
;
; Call marquardt
marquardt,'orbfuncs',jd,r,sig,ndata,a,ia,ma,chisq,covar
;
; Set new paramters
o_parms(0:6)=a(0:ma-1)
RAD=180/pi_circle
print,'__________________________________'
print,'Semi-major axis = ',o_parms(0)
print,'Eccentricity =    ',o_parms(1)
print,'Inclination =     ',o_parms(2)*RAD
print,'Periastron =      ',o_parms(3)*RAD
print,'Ascending node =  ',o_parms(4)*RAD
print,'Period =          ',o_parms(5)
print,'Epoch =           ',o_parms(6)
print,'_______________***________________'
;
end
;************************************************************************Block 4
pro marquardt,func,x,y,sig,ndata,a,ia,ma,chisq,covar,alamda
;
; Iterative procedure to do non-linear least-squares fit.
;
common MarquardtFit,fit_options
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
;
if n_elements(fit_options) eq 0 then if init_marquardtfit() ne 0 then return
;
chifr=fit_options.chifr
tol=fit_options.tolerance
alamda=-fit_options.alamda
;
chisq=0.d0
marquardtmin,x,y,sig,ndata,a,ia,ma,covar,alpha,chisq,func,alamda,tol
print,''
print,'Initial chisq=',chisq
;
i=1
repeat begin
	chisqo=chisq
	marquardtmin,x,y,sig,ndata,a,ia,ma,covar,alpha,chisq,func,alamda,tol
	print,'Iteration ',i,' completed; chisq = ',chisq
	i=i+1
endrep until (1-chisq/chisqo lt chifr and chisq lt chisqo) $
	or alamda gt 1 $
	or i gt 20 $
	or fit_options.o
print,'Final   chisq=',chisq,', reduced = ',chisq/n_elements(y)
print,'Iter=',i,', Alamda=',alamda,format='(a,i2,a,f8.3)'
;
alamda=0.d0
marquardtmin,x,y,sig,ndata,a,ia,ma,covar,alpha,chisq,func,alamda,tol
;
end
;-------------------------------------------------------------------------------
function fixell,r,t,a,ex,ey
;
dx=ex-a(0)
dy=ey-a(1)
r=sqrt(dx^2+dy^2)
t=atan(dx,dy)-a(4)
esqr=1-(a(3)/a(2))^2
return,ellipse(a(2),a(3),t)
end
;-------------------------------------------------------------------------------
pro ellfuncs,p,r,a,rmod,drda,ma,ndata,ia
;
common FitEllipse,ex,ey
;
deltaa=dblarr(5)
deltaa(0)=0.1	; Center x
deltaa(1)=0.1	; Center y
deltaa(2)=0.1	; Major axis
deltaa(3)=0.1	; Minor axis
deltaa(4)=0.02	; Position angle
;
drda=dblarr(ndata,ma)
;
b=a
;
for j=0,ma-1 do begin
	if ia(j) gt 0 then begin
		b(j)=a(j)-deltaa(j)
		r_low=fixell(r,p,b,ex,ey)
		r_low=r_low-r
		b(j)=a(j)+deltaa(j)
		r_high=fixell(r,p,b,ex,ey)
		r_high=r_high-r
		b(j)=a(j)
		drda(*,j)=(r_high-r_low)/(2*deltaa(j))
	endif
endfor
;
rmod=fixell(r,p,a,ex,ey)
;
end
;-------------------------------------------------------------------------------
pro orbfuncs,jd,a,rmod,drda,ma,ndata,ia
;
common FitEllipse,ex,ey
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
;
deltaa=dblarr(7)
deltaa(0)=0.1			; Major axis increment
deltaa(1)=0.02			; Eccentricity
deltaa(2)=0.02			; Inclination
deltaa(3)=0.005/(a(1)^2+0.005)	; Periastron
deltaa(4)=0.02/(sin(a(2))+0.02)	; Node
deltaa(5)=a(5)/1000		; Period
deltaa(6)=a(5)/1000		; Epoch
;
if n_elements(gen_model) ne 0 then begin
	ra=gen_model.ra
	dec=gen_model.dec
endif
;
drda=dblarr(ndata,ma)
;
b=a
;
for j=0,ma-1 do begin
	if ia(j) gt 0 then begin
		b(j)=a(j)-deltaa(j) & bb=[b(0:3),0,b(4:6)]
		xy=true2app(jd,bb,rho,theta,gen_model.ra,gen_model.dec)
		r_low=sqrt((xy(*,0)-ex)^2+(xy(*,1)-ey)^2)
		b(j)=a(j)+deltaa(j) & bb=[b(0:3),0,b(4:6)]
		xy=true2app(jd,bb,rho,theta,ra,dec)
		r_high=sqrt((xy(*,0)-ex)^2+(xy(*,1)-ey)^2)
		b(j)=a(j)
		drda(*,j)=(r_high-r_low)/(2*deltaa(j))
	endif
endfor
;
aa=[a(0:3),0,a(4:6)]
xy=true2app(jd,aa,rho,theta,gen_model.ra,gen_model.dec)
rmod=sqrt((xy(*,0)-ex)^2+(xy(*,1)-ey)^2)
;
end
;-------------------------------------------------------------------------------
pro modfuncs,x,y,a,ymod,ysig,dyda,ma,ndata,ia
;
; Store model parameters and get step sizes
modelfitparmcopy,1,a,deltaa
;
; Compute derivatives
b=a
dyda=dblarr(ndata,ma)
for j=0,ma-1 do begin
	if ia(j) gt 0 then begin
		b(j)=a(j)-deltaa(j)
		modelfitparmcopy,1,b
		calcmodel
		marquardtdata,mrq_y,mrq_ysig,y_low
		b(j)=a(j)+deltaa(j)
		modelfitparmcopy,1,b
		calcmodel
		marquardtdata,mrq_y,mrq_ysig,y_high
		dyda(*,j)=(y_high-y_low)/(2*deltaa(j))
		b(j)=a(j)
	endif
endfor
;
; Compute model
modelfitparmcopy,1,a
calcmodel
marquardtdata,y,ysig,ymod
;
; Astrometry fits vary sigma based on the orientation of the uncertainty 
; ellipse wrt to the model position
;
end
;-------------------------------------------------------------------------------
pro funct_gauss,x,a,f
;
; Gauss fit, a(2) is FWHM
;
exponents=-(2*sqrt(alog(2))*(x-a(1))/(a(2)))^2
index=where(exponents lt -30,count)
if count gt 0 then exponents(index)=-30
f=a(0)*exp(exponents)
;
end
;-------------------------------------------------------------------------------
function mask2pix,winv,lambda,a,flux=flux
;
; Given a polynomial describing the position of the mask, as well
; as width information, fit mask profiles to each column and return
; the values for an evaluation of how well this masks follows the
; spectrum.
;
; x0 is the reference pixel for the polynom definition: x0(PRISM)=130
; a=[a0,a1,a2], polynomial fit coefficients, a3=width in pixels
;
n=n_elements(winv)
nx=n_elements(lambda)
ny=n/nx
wina=reform(winv,nx,ny)
winf=wina
x=findgen(nx)
y=findgen(ny)
x0=a(4)
y0=poly(x-x0,a(0:2))
; w=lambda*0+a(3)
w=a(3)*lambda/lambda(fix(x0))
flux=fltarr(nx)
;
for j=0,nx-1 do begin
	p=reform(wina(j,*))
	gp=[max(p),y0(j),w(j)]
	winf(j,*)=curvefit(y,p,p*0+1,gp,fita=[1,0,0], $
		function_name='funct_gauss',/noderivat)
	flux(j)=gp(0)
endfor
;
return,reform(winf,n)
;
end
;-------------------------------------------------------------------------------
pro maskfuncs,x,y,a,ymod,ysig,dyda,ma,ndata,ia
;
common FitMask,lambda,x0,pho_file,det_window,coeffs,num_bad_col,num_bad_row
;
; a: a0,a1,a2: polynomial coefficients, a3=width at x0
;
a(3)=abs(a(3))		; width must not become negative
;
deltaa=dblarr(5)
deltaa(0)=0.5		; a0, the position at 130 pixels
deltaa(1)=0.02		; a1, linear term
deltaa(2)=0.00005	; a2, quadratic term
deltaa(3)=0.1		; width
deltaa(4)=1.0		; This position corresponds to x0, which is never fit.
;
dyda=dblarr(ndata,ma)
;
b=a
;
for j=0,ma-1 do begin
	if ia(j) gt 0 then begin
		b(j)=a(j)-deltaa(j)
		y_low=mask2pix(y,lambda,b)
		b(j)=a(j)+deltaa(j)
		y_high=mask2pix(y,lambda,b)
		b(j)=a(j)
		dyda(*,j)=(y_high-y_low)/(2*deltaa(j))
	endif
endfor
;
ymod=mask2pix(y,lambda,a)
;
end
;************************************************************************Block 5
pro setorbit
;
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
RAD=180/pi_circle
;
if n_elements(binary_model) eq 0 then begin
	print,'***Error(SETORBIT): no binary_model loaded!'
	return
endif
;
index=where(binary_model.component eq orbit_options.component,count)
if count eq 0 then begin
	print,'***Error(SETORBIT): component not found in binary_model!'
	return
endif
o_parms=dblarr(8)
o_parms(0)=binary_model(index).semimajoraxis
o_parms(1)=binary_model(index).eccentricity
o_parms(2)=binary_model(index).inclination/RAD
o_parms(3)=binary_model(index).periastron/RAD
o_parms(4)=binary_model(index).ascendingnode/RAD
o_parms(5)=binary_model(index).period
o_parms(6)=binary_model(index).epoch
;
print,'Orbit set.'
;
end
;-------------------------------------------------------------------------------
pro setmodel,star
;
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
common FitAstrometry,ellipse_options,orbit_options,e_parms,o_parms
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
common StarBase,StarTable,Notes
;
RAD=180/pi_circle
;
if n_elements(binary_model) eq 0 then begin
	print,'***Warning(SETMODEL): initializing model...'
;
	wavelengths=[0.550d0,0.700d0,0.850d0]
	num_wave=n_elements(wavelengths)
	gen_model=alloc_gen_model(num_wave)
	gen_model.starid='CCCNNNN'
	gen_model.wavelengths=wavelengths
	gen_error=gen_model
;
	star_struct=alloc_star_struct(num_wave)
	star_model=replicate(star_struct,2)
	star_model(0).component='A'
	star_model(1).component='B'
	star_model(*).mass=1
	star_error=star_model
;
	binary_struct=alloc_binary_struct()
	binary_model=replicate(binary_struct,1)
	binary_model(0).component='A-B'
	binary_model(0).mode=1
	binary_error=binary_model
;
endif
;
if n_elements(star) ne 0 then begin
	index=where(StarTable.starid eq star,count)
	if count eq 0 then return
	gen_model.starid=star
	gen_model.ra=startable(index).ra
	gen_model.dec=startable(index).dec
	star_model.mass=1
        binary_model.semimajoraxis=startable(index).a
        binary_model.inclination=startable(index).i
        binary_model.ascendingnode=startable(index).n
        binary_model.eccentricity=startable(index).e
        binary_model.periastron=startable(index).o
        binary_model.apsidalmotion=0
        binary_model.epoch=startable(index).t-2440000L
        binary_model.period=startable(index).p
	return
; 
endif
;
index=where(binary_model.component eq orbit_options.component,count)
if count eq 0 then begin
	print,'***Error(SETMODEL): component not found in binary_model!'
	return
endif
binary_model(index).semimajoraxis	=o_parms(0)
binary_model(index).eccentricity	=o_parms(1)
binary_model(index).inclination		=o_parms(2)*RAD
binary_model(index).periastron		=o_parms(3)*RAD
binary_model(index).ascendingnode	=o_parms(4)*RAD
binary_model(index).period		=o_parms(5)
binary_model(index).epoch		=o_parms(6)
;
print,'Model set.'
;
end
;************************************************************************Block 5
pro fitnights
;
; Do interferometry fits (usually including separation and position angle,
; possibly with diameters and magnitude differences) to single nights, 
; obtaining initial estimates for rho and theta from the global model. 
; The fit parameters must be selected through the widgets.
;
common MarquardtFit,fit_options
common ModelFit,parameters,ds_options
common ScanData,scans,bgscans,bufferinfo,positions,velocities,magnitudes
common DataSelInfo,class,type,slice,ds_nights,ds_stars,ds_x,ds_y,ds_z,ps_options
common SysConfig,SystemId,Date,MetroConfig,GenConfig,GeoParms,GenInfo,GeoInfo
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
common Model,gen_model,star_model,binary_model,gen_error,star_error,binary_error
;
RAD=180/pi_circle
;
; Save geoinfo, geninfo, and bufferinfo
geoinfo_bck=geoinfo
geninfo_bck=geninfo
bufferinfo_bck=bufferinfo
;
; The nights are selected in the plot widget
if n_elements(ds_nights) eq 0 then plotinit=init_plot('amoeba','pt')
if set_nightsel() ne 0 then return
;
; Save other current settings
binary_model_bck=binary_model
modes=binary_model.mode
weight_i=ds_options.i
weight_s=ds_options.s
weight_p=ds_options.p
weight_a=ds_options.a
if n_elements(fit_options) eq 0 then if init_marquardtfit() ne 0 then return
chifr=fit_options.chifr
;
; Make sure we find the minimum
fit_options.chifr=0.001
;
; Change some settings
case SystemId of
        'NPOI':  midnight=7.0
        'Mark3': midnight=8.0
        else:    midnight=8.0
endcase
ds_options.i=1
ds_options.s=0
ds_options.p=0
ds_options.a=0
;
; Open results file
openw,unit,'fitnights.psn',/get_lun
;
for i=0,n_elements(ds_nights)-1 do begin
	print,ds_nights(i)
;
	j=where(geninfo_bck.date+' '+geninfo_bck.configid eq ds_nights(i))
;	This ensures only this night is used
	geoinfo=geoinfo_bck(j)
	geninfo=geninfo_bck(j)
	bufferinfo=bufferinfo_bck(j)
	parsedate,ds_nights(i),y,m,d
	d=d+midnight/24
	epoch_jd=julian(y,m,d)
	epoch_jy=jd2jy(epoch_jd)
;	Obtain predictions for (rho,theta) for this night
	binary_model.mode=modes
	for j=0,num_binary()-1 do begin
;		These are the center-of-mass coordinates
		rt=binarypos(epoch_jd,binary_model(j).component,/com)
		binary_model(j).rho=rt(0)
		binary_model(j).theta=rt(1)
	endfor
	binary_model.mode=2
;	For those components which have orbital elements include orb. motion
	index=where(modes eq 1,count)
	if count gt 0 then binary_model(index).mode=3
;	Fit model
	fitmodel,chisq,ndata
	binary_model.mode=2
	locations=replicate(location(),num_binary())
;
	FOR j=0,num_binary()-1 DO BEGIN
;
	locations(j).jy=epoch_jy
	locations(j).component=binary_model(j).component
;	These are now the photocenter coordinates
	rt=binarypos(epoch_jd,binary_model(j).component)
	locations(j).rho=rt(0)
	locations(j).theta=rt(1)/RAD
	if locations(j).theta lt 0 then locations(j).theta= $
					locations(j).theta+2*pi_circle
	rho_error=binary_error(j).rho
	theta_error=binary_error(j).theta
	chisq_data=chisq
;	
;	Compute CLEAN beam size
	e_parms=cleanbeam(dirtybeam(gen_model.starid))
	e_parms(0)=e_parms(0)/2	
	e_parms(1)=e_parms(1)/2
	e_parms(2)=e_parms(2)/RAD
;
; 	Estimate calibrated uncertainty ellipse
	index=where(scans.starid eq gen_model.starid)
	v=scans(index).vissqc
	e=scans(index).vissqcerr
	m=scans(index).vissqm
	index=where(e gt 0,count)
	rms=stdev(v(index)-m(index))
	sig=median(e(index))
	f=(sqrt(2)*0.3)/((max(([rms,sig]))/(2*sqrt(2)))/sqrt(count))
	map_rms=(max([rms,sig])/2)/sqrt(count)
	f=0.5/(map_rms*0.6)
;	We set f since we do not quite believe this result
	f=1.0
	locations(j).emajor=e_parms(0)/f
	locations(j).eminor=e_parms(1)/f
	locations(j).pa=e_parms(2)
;
;	Adjust uncertainty ellipse to equal a total increase of 1*reduced chisq
if 0 then begin
	x=locations(j).rho*sin(locations(j).theta) $
	 -locations(j).emajor*sin(locations(j).pa)
	y=locations(j).rho*cos(locations(j).theta) $
	 -locations(j).emajor*cos(locations(j).pa)
	binary_model(j).rho=sqrt(x^2+y^2)
	binary_model(j).theta=atan(x,y)*RAD
	calcmodel
	a=(modelchisq()-chisq)/(locations(j).emajor)^2
	ff1=locations(j).emajor/sqrt((max([chisq,1])/ndata)/a)
	x=locations(j).rho*sin(locations(j).theta) $
	 -locations(j).eminor*cos(locations(j).pa)
	y=locations(j).rho*cos(locations(j).theta) $
	 +locations(j).eminor*sin(locations(j).pa)
	binary_model(j).rho=sqrt(x^2+y^2)
	binary_model(j).theta=atan(x,y)*RAD
	calcmodel
	a=(modelchisq()-chisq)/(locations(j).eminor)^2
	ff2=locations(j).eminor/sqrt((max([chisq,1])/ndata)/a)
	locations(j).emajor=e_parms(0)/(f*ff1)
	locations(j).eminor=e_parms(1)/(f*ff2)
endif
;
	r=ellipse(locations(j).emajor, $
	          locations(j).eminor, $
	          locations(j).theta-e_parms(2))
	factor=(locations(j).rho/100)/r
	if factor gt 1 and geoinfo.SystemId eq 'not_obsolete' then begin
		d_pa=e_parms(2)-locations(j).theta
		if d_pa lt 0 then d_pa=d_pa+!pi
		new_parms=apollonius(locations(j).emajor, $
				     locations(j).eminor, $
				     d_pa,factor)
		locations(j).emajor=new_parms(0)
		locations(j).eminor=new_parms(1)
		locations(j).pa=(new_parms(2)+locations(j).theta) mod !pi
	endif
;
; 	Write results to file
	printf,unit,locations(j).component,locations(j).jy, $
	      	locations(j).rho,locations(j).theta*RAD, $
		locations(j).emajor,locations(j).eminor,locations(j).pa*RAD, $
		rho_error,theta_error,chisq, $
		format='(a4,2x,f9.4,2x,f6.2,1x,f7.2,2x,f5.3,1x,f5.3,1x,f6.1,'+ $
		       	'3x,f6.4,1x,f7.4,1x,f4.1)'
;
	ENDFOR
;
endfor
;
; Close file
free_lun,unit
;
; Set some of the parms back to old values
binary_model=binary_model_bck
geoinfo=geoinfo_bck
geninfo=geninfo_bck
bufferinfo=bufferinfo_bck
;
ds_options.i=weight_i
ds_options.s=weight_s
ds_options.p=weight_p
ds_options.a=weight_a
fit_options.chifr=chifr
;
end
;-------------------------------------------------------------------------------
