;*******************************************************************************
; File: functions.pro
;
; Description:
; ------------
; This file contains various functions for the CHAMELEON/AMOEBA software.
;
; Block directory:
; ----------------
; Block 1: fakultaet,combinations,eqp,produkt,summe,add,box,kreuzprodukt,
;	   histogramm,histograph,
;	   wmean,medianve,gavg,ravg1,ravg2,gsmooth,
; 	   signof,cphase,cmag,hb,bessjy8,sinc,quasol,winkel,
;	   ellipse,apollonius,
;	   evalfunction,polynom,polyfit,
;	   signalfilter,
;	   stringl,myhak,elements,printindex,isnumeric,gitter,mergetable,
;	   sizeof,natural,gapindex,
;	   edlen,owens,oneill,silica,extinct,blackbody,jy2n,n2jy
;	   getcolor,getsymbol,greek
; Block 2: poweroftwo,powerpatch,powerspectrum,strucfunc,
;	   siggen,signalfilter
; Block 3: stringparse,retroparse,nameparse,criparse,cri,hms,dms,hms2h,
;	   finddir,npoifile,pathname,specname,safe,breve
; Block 4: set_boxes,set_points,get_ellipse,set_ellipse
;
;************************************************************************Block 1
function fakultaet,n
;
; Return factorial of n.
;
f=1L
for i=2,n do f=f*i
return,f
end
;-------------------------------------------------------------------------------
function combinations,n,i
;
; Return number of combinations with i samples taken from n elements.
;
c=1L
for j=n-i+1,n do c=c*j
return,c/fakultaet(i)
end
;-------------------------------------------------------------------------------
function eqp,m,n
;
; Return the probability of n counts for a Poisson process with mean m.
; Input n can be an array.
;
md=double(m)
nd=double(n)
y=dblarr(n_elements(n))
for i=0,n_elements(n)-1 do begin
	p=-md/max([nd(i),1.d0])
	if p lt -40 then a=0.d0 else a=exp(p)
	case n(i) of
      0:y(i)=a
   else:begin
	y(i)=md*a
	for k=2,n(i) do y(i)=y(i)*md/double(k)*a
	end
	endcase
endfor
;
return,y
;
end
;-------------------------------------------------------------------------------
function produkt,values
;
; Return the product of all values.
;
p=1L
for i=0,n_elements(values)-1 do p=p*values(i)
return,p
end
;-------------------------------------------------------------------------------
function summe,x,dim,sdev=sdev
;
; This function sums all elements along dimension dim (first is 0) and
; returns the remainder of the matrix. It is identical to total() in IDL,
; but should be used instead of total() under PV-WAVE since that 
; implementation does not have the dim parameter.
;
; If the input array is complex, the standard deviations (sdev) are 
; computed as the standard deviations of the radius and phase*radius,
; i.e. as the major and minor axes of the uncertainty ellipse after 
; rotation by the mean phase angle to phase=0. 
;
forward_function cphase
;
i_complex=sqrt(complex(-1,0))
;
result=size(x)
if result(n_elements(result)-2) eq 6 then c=1 else c=0
if n_elements(dim) eq 0 then begin
	xt=total(x)
	if n_elements(x) ge 2 then begin
	if c then begin
		xr=x*exp(-i_complex*cphase(xt))
		sdev=complex(stdev(float(xr)),stdev(imaginary(xr)))
;		sdev=complex(stdev(float(xr)), $
;		             atan(stdev(imaginary(xr)),abs(xt)/n_elements(x)))
	endif else sdev=stdev(x)
	endif
	return,total(x)
endif
;
n=result(1+dim)
if dim eq 0 then k=1 else k=produkt(result(1:dim))
l=k*n
result(dim+1)=1
xt=reform(make_array(size=result))
sdev=xt
m=n_elements(xt)
for i=0L,m-1 do begin
	index=lindgen(n)*k+(i mod k)+(i/k)*l
	xt(i)=total(x(index))
	if n ge 2 then begin
	if c then begin
		xr=x(index)*exp(-i_complex*cphase(xt(i)))
		sdev(i)=complex(stdev(float(xr)),stdev(imaginary(xr)))
;		sdev(i)=complex(stdev(float(xr)), $
;			 atan(stdev(imaginary(xr)),abs(xt(i))/n))
	endif else sdev(i)=stdev(x(index))
	endif
endfor
;
return,xt
;
end
;-------------------------------------------------------------------------------
function add,x,n,dim,sdev=sdev
;
; Sum every n samples along dimension dim (first is 0) of a matrix x.
; If the requested dimension is larger than the number of dimensions
; of x, it defaults to the last dimension of x.
; 
if n_elements(dim) eq 0 then dim=0
;
if n eq 1 then return,x
;
r=size(x)
dim=dim < (r(0)-1)
k=r(1+dim)/n
r0=r
r0(1+dim)=k
result=make_array(size=r0)
sdev=make_array(size=r0)
;
r1=r
r1(1+dim)=1
m=produkt(r1(1:r(0)))
if dim eq 0 then m1=1 else m1=produkt(r1(1:dim))
m2=m1*k
index=(lindgen(m)/m1)*m2+(lindgen(m) mod m1)
;
r2=r
r2(1+dim)=n
m2=produkt(r(1:dim+1))
jndex=(lindgen(m*n)/(m1*n))*m2+(lindgen(m*n) mod (m1*n))
;
for i=0L,k-1 do begin
	result(index+i*m1)=summe(reform(x(jndex+i*m1*n),r2(1:r(0))),dim,sdev=sd)
	sdev(index+i*m1)=sd
endfor
;
return,result
;
end
;-------------------------------------------------------------------------------
function box,x,j,dim
;
; Smooth values along dimension dim (first is 0) of a matrix x, over j samples.
;
if n_elements(dim) eq 0 then dim=0
;
if j eq 1 then return,x
;
result=size(x)
;
; n is the number of elements along dim
n=result(1+dim)			
;
; k is the number of elements in each of the n sub-matrices 
if dim eq 0 then k=1 else k=produkt(result(1:dim))
;
; l is the number of elements processed at a time
l=k*n
;
; Make output array
result=reform(make_array(size=result))
;
; m is the number of elements not including dim
m=n_elements(result)/n
;
for i=0L,m-1 do begin
	index=lindgen(n)*k+(i mod k)+(i/k)*l
	if j eq n_elements(index) $
		then result(index)=mean(x(index)) $
		else result(index)=smooth(x(index),j,/edge_truncate)
endfor
;
return,result
;
end
;-------------------------------------------------------------------------------
function kreuzprodukt,x,y,normal=normal
;
if n_elements(normal) eq 0 then normal=0 else normal=normal gt 0
v=[x(1)*y(2)-x(2)*y(1),x(2)*y(0)-x(0)*y(2),x(0)*y(1)-x(1)*y(0)]
if normal then v=v/sqrt(total(v^2))
return,v
;
end
;-------------------------------------------------------------------------------
function histogramm,x,y,min=minx,max=maxx,binsize=binsize,values=values
;
; Places values x in bins similar to histogram, but averages corresponding
; values y and returns array of averages.
;
if n_elements(minx) eq 0 then minx=min(x)
if n_elements(maxx) eq 0 then maxx=max(x)
if n_elements(binsize) eq 0 then binsize=1
;
;
f=histogram(x,min=minx,max=maxx,binsize=binsize,reverse_indices=r)
n=n_elements(f)
values=findgen(n)*binsize+minx+binsize/2
if n_elements(y) eq 0 then return,f
;
y_mean=fltarr(n)
for i=0,n-1 do begin
	if r(i) ne r(i+1) then $
	y_mean(i)=avg(y(r(r(i):r(i+1)-1)))
endfor
return,y_mean
;
for i=0,n-2 do begin
        index=where(x ge values(i) and x lt values(i+1),count)
        if count gt 0 then y_mean(i)=avg(y(index))
endfor
return,y_mean
;
end
;-------------------------------------------------------------------------------
pro histograph,y,min=miny,max=maxy,binsize=binsize,oplot=oplot,color=color
;
if n_elements(miny) eq 0 then miny=min(y)
if n_elements(maxy) eq 0 then maxy=max(y)
if n_elements(binsize) eq 0 then binsize=1
if n_elements(color) eq 0 then color=1
;
f=histogram(y,min=miny,max=maxy,binsize=binsize)
n=n_elements(f)
x=findgen(n)*binsize+miny
x2=fltarr(2*n)
x2(indgen(n)*2)=x
x2(indgen(n)*2+1)=x+binsize
f2=fltarr(2*n)
f2(indgen(n)*2)=f
f2(indgen(n)*2+1)=f
if keyword_set(oplot) then oplot,x2,f2,psym=0,color=color $
		      else plot,x2,f2,psym=0
;
end
;-------------------------------------------------------------------------------
function wmean,y,e,me,chisq
;
w=1/e^2
tw=total(w)
ym=total(y*w)/tw
me=sqrt(1/tw)
chisq=total((y-ym)^2*w)/n_elements(y)
;
return,ym
;
end
;-------------------------------------------------------------------------------
function medianve,y,e
;
n=n_elements(y)
if n eq 0 then return,0
si=sort(y)
ys=y(si)
if n mod 2 eq 1 then v=ys((n-1)/2) else v=(ys(n/2)+ys(n/2-1))/2
if n gt 3 then e=(ys(n/2+n/3)-ys(n/2-n/3))/2
;
return,v
end
;-------------------------------------------------------------------------------
function gavg,values
;
; Return a geometric mean of all values
;
index=where(values eq 0,count)
if count gt 0 then return,0
index=where(values lt 0,count)
if count gt 0 then print,'Warning(GAVG): negative values, use absolut instead!'
return,exp(avg(alog(abs(values))))
end
;-------------------------------------------------------------------------------
function ravg1,y,w
;
; Return running average. Initialize with non-zero value of width (int).
; After initialization, w will be zero and the functions returns the
; average of the last w values of y, including the current value of y.
;
common LocalRavg1,width,num,buffer,current
;
if n_elements(w) eq 0 then w=0
;
if w gt 0 then begin
	width=w
	buffer=fltarr(width)
	num=0
	current=0.0
	w=0
endif
;
if num eq width then begin
	current=current-buffer(0)
	buffer=shift(buffer,-1)
endif
num=min([width,num+1])
current=current+y
buffer(num-1)=y
return,current/num
;
end
;-------------------------------------------------------------------------------
function ravg2,y,w
;
; Return running average.
;
common LocalRavg2,width,num,buffer,current
;
if n_elements(w) eq 0 then w=0
;
if w gt 0 then begin
	width=w
	buffer=fltarr(width)
	num=0
	current=0.0
	w=0
endif
;
if num eq width then begin
	current=current-buffer(0)
	buffer=shift(buffer,-1)
endif
num=min([width,num+1])
current=current+y
buffer(num-1)=y
return,current/num
;
end
;-------------------------------------------------------------------------------
function gsmooth,t,y,s,x,c8=c8
;
if n_elements(c8) eq 0 then c8=0
;
if c8 then begin
	t8=double(t)
	y8=double(y)
	s8=double(s)
	n=n_elements(y8)
	r8=dblarr(n)
	status=linknload(!external_lib,'gsmooth8',t8,y8,s8,n,r8)
	r=float(r8)
endif else begin
	r=y
	a=sqrt(10)*s
	for i=0l,n_elements(t)-1 do begin
		dt=abs(t-t(i))
		index=where(dt lt a)
		w=exp(-(dt(index)/s)^2)
		r(i)=total(y(index)*w)/total(w)
	endfor
endelse
;
if n_elements(x) ne 0 then return,interpol(r,t,x) else return,r
;
end
;-------------------------------------------------------------------------------
function signof,x,y
;
if n_elements(y) eq 0 then return,(x ge 0)*2-1 $
		      else return,(x ge 0 and y ge 0)*2-1
end
;-------------------------------------------------------------------------------
function nonzero,x
;
index=where(float(x) eq 0.0,count)
if count gt 0 then x(index)=1
;
return,x
;
end
;-------------------------------------------------------------------------------
function cphase,c
;
; Return the phase of a complex variable
;
cr=float(c)
ci=imaginary(c)
index=where(cr eq 0 and ci eq 0,count)
p=atan(imaginary(c),float(c))
if count gt 0 then p(index)=0
return,p
;
end
;-------------------------------------------------------------------------------
function cmag,m1,m2
;
; Compute the combined magnitude of m1 and m2.
;
f1=fltarr(n_elements(m1))
index=where(m1 ne +100,count)
if count gt 0 then f1(index)=10.^(-m1(index)/2.5)
;
f2=fltarr(n_elements(m2))
index=where(m2 ne +100,count)
if count gt 0 then f2(index)=10.^(-m2(index)/2.5)
;
f=f1+f2
mag=fltarr(n_elements(f))+100
index=where(f ne 0,count)
if count gt 0 then mag(index)=-2.5*alog10(f(index))
if count eq 1 then return,mag(0) else return,mag
;
end
;-------------------------------------------------------------------------------
function dmag,mt,dm
;
; Compute m1 and m2 from the total magnitude and the magnitude difference.
;
ft=fltarr(n_elements(mt))
index=where(mt ne +100,count)
if count gt 0 then ft(index)=10.^(-mt(index)/2.5)
;
fr=fltarr(n_elements(dm))
index=where(dm ne +100,count)
if count gt 0 then fr(index)=10.^(-dm(index)/2.5)
;
m1=fltarr(n_elements(mt))+100
m2=fltarr(n_elements(mt))+100
index=where(fr ne 0,count)
if count gt 0 then begin
	f2=ft/(1+fr)
	f1=f2*fr
	m1(index)=-2.5*alog10(f1)
	m2(index)=-2.5*alog10(f2)
endif
print,m1,m2
;
end
;-------------------------------------------------------------------------------
function hb,uv,v2
;
; This function returns for a given uv spacing (in millions of wavelengths)
; and calibrated squared visibility amplitude the diameter of the star [mas]. 
; The computation is non-iterative, since it uses polynomial fit coefficients
; determined for a grid of spacings and visibilities. The function works
; only reliably if v2 > 0.1 and v2 < 0.9.
;
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
factor=(pi_circle^2)/(180L*3600L*1000L)
;
; These coefficients are for a 3rd order fit of the d=f(v2)
; coefficients to the uv spacings.
coeff11=[10.8555,-1.52880,0.0975650,-0.00215141]
coeff21=[3.37067,-0.0641237,0.000618756,-2.44172e-06]
coeff12=[-12.2294,1.49343,-0.0793799,0.00141445]
coeff22=[-3.75824,0.0642599,-0.000620554,2.45088e-06]
coeff13=[11.6333,-1.21883,0.0499391,-0.000526415]
coeff23=[3.73860,-0.0645361,0.000624810,-2.47455e-06]
coeff14=[10.3883,-1.14758,0.0501068,-0.000630298]
coeff24=[3.23568,-0.0646144,0.000626390,-2.48441e-06]

;
; Get polynomial coefficients for given uv distance. There are two sets
; because of a discontinuity at 18 Mlambda.
if uv le 18 then r=exp([poly(uv,coeff11),-poly(uv,coeff12), $
                        poly(uv,coeff13),poly(uv,coeff14)]) $
            else r=exp([poly(uv,coeff21),-poly(uv,coeff22), $
                        poly(uv,coeff23),poly(uv,coeff24)])
;
; Get diameter for given uv and v2
d=poly(v2,r*[1,-1,1,-1])
;
return,d
;
; Test
uvdist=uv*1e6
arg=factor*uvdist*d
visamp=2*beselj(arg,1)/arg
vissq=visamp^2
print,'V2 should be ',vissq
;
end
;-------------------------------------------------------------------------------
function bessjy8,arg,arg_nu
;
; Double-precision fractional bessel function.
;
;
x=double(arg)
x_nu=double(arg_nu)
n=n_elements(x)
rj=x
status=linknload(!external_lib,'bessjyidl',n,x,x_nu,rj)
;
return,rj
;
end
;-------------------------------------------------------------------------------
function spline8,x,y,t
;
x=double(x)
y=double(y)
y2=y
t=double(t)
r=t
n=n_elements(x)
m=n_elements(t)
;
status=linknload(!external_lib,'splineint',x,y,y2,t,r,n,m)
;
return,r
;
end
;-------------------------------------------------------------------------------
function sinc,x_in
;
x=x_in
index=where(x eq 0,count)
if count gt 0 then x(index)=1
y=sin(!pi*x)/(!pi*x)
if count gt 0 then y(index)=1
return,y
;
end
;-------------------------------------------------------------------------------
function quasol,c
;
d=sqrt(c(1)^2-4*c(2)*c(0))
s1=(-c(1)+d)/(2*c(2))
s2=(-c(1)-d)/(2*c(2))
;
return,[s1,s2]
;
end
;-------------------------------------------------------------------------------
function winkel,ra1,dec1,ra2,dec2
;
; Given two positions on the sky, return the angle between them in degrees.
rad=180/!pi
ra1rad=ra1*15/rad
ra2rad=ra2*15/rad
dec1rad=dec1/rad
dec2rad=dec2/rad
;
return,acos(cos(dec1rad)*cos(dec2rad)*cos(ra1rad-ra2rad) $
		+sin(dec1rad)*sin(dec2rad))*rad
;
end
;-------------------------------------------------------------------------------
function ellipse,sma,smb,pa
;
; Return radius of point on ellipse at position angle pa. Origin is at
; center of ellipse; radius is largest at p=0. pa in radians. a and b 
; are the semi-major and minor axes.
;
a=float(sma)
b=float(smb)
p=float(pa)
;
esqr=1-(b/a)^2
return,sqrt(b^2/(1-esqr*cos(p)^2))
;
end
;-------------------------------------------------------------------------------
function apollonius,sma,smb,pa,sfactor
;
; pa is the position angle (positive sense) in radians.
;
a=float(sma)
b=float(smb)
p=float(pa)
factor=float(sfactor)
;
q=0
if p gt !pi/2 and p le !pi then begin
        q=1
        p=!pi-p
endif else if p gt !pi then begin
        q=2
        p=p-!pi
endif else if p lt 0 and p ge -!pi/2 then begin
        q=3
        p=-p
endif else if p lt -!pi/2 then begin
        q=4
        p=p+!pi
endif
;
a1=ellipse(a,b,!pi/2-p)
b1=sqrt(a^2+b^2-a1^2)
;
x1=b1*sqrt(1-((a*b)/(a1*b1))^2)
y1=((a*b)/(a1*b1))*b1
;
x2=x1
y2=y1*factor
;
a2=a1
b2=sqrt(x2^2+y2^2)
;
ab=a2*b2*(y2/b2)
asqplusbsq=a2^2+b2^2
;
af=sqrt((asqplusbsq+sqrt(asqplusbsq^2-4*ab^2))/2)
bf=sqrt((asqplusbsq-sqrt(asqplusbsq^2-4*ab^2))/2)
;
if x2 ne 0 then begin
        tanalphaplusbeta=y2/x2
        tanalphatimestanbeta=bf^2/af^2
        t1=tanalphaplusbeta*(1-tanalphatimestanbeta)
        t2=tanalphatimestanbeta
        alpha=atan((t1+sqrt(t1^2-4*t2))/2)
        beta =atan((t1-sqrt(t1^2-4*t2))/2)
endif else begin
        alpha=!pi/2
        beta =0
endelse
;
if factor lt 1 then paf=!pi/2-beta else paf=!pi/2-alpha
;
case q of
        0:paf=paf
        1:paf=!pi-paf
        2:paf=paf+!pi
        3:paf=-paf
        4:paf=paf-!pi
endcase
;
return,[af,bf,paf]
;
end
;-------------------------------------------------------------------------------
function evalfunction,term,x,x_mid,x_scl
;
; This is the one and only function to evaluate mathematical base functions
; used for calibration and fitting of data. See create_calentries for 
; initialization of function terms.
;
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
if n_elements(x_mid) eq 0 then x_mid=0.0
if n_elements(x_scl) eq 0 then x_scl=1.0
;
xx=(x-x_mid)*x_scl
case term of
;	Polynomial
	'1'  : 	y=x^0
	'x'  : 	y=x
	'x^2': 	y=x^2 
	'x^3': 	y=x^3 
	'x^4': 	y=x^4
	'x^5': 	y=x^5
	'x^6': 	y=x^6
;	Legendre polynomial
        'L_0': 	y=xx^0
        'L_1': 	y=xx
        'L_2': 	y=(3*xx^2-1)/2
        'L_3': 	y=(5*xx^3-3*xx)/2
        'L_4': 	y=(35*xx^4-30*xx^2+3)/8
        'L_5': 	y=(63*xx^5-70*xx^3+15*xx)/8
        'L_6': 	y=(231*xx^6-315*xx^4+105*xx^2-5)/16
;	Phase change due to silica and air path mismatches = coeff/360 in microns
	'P_0':  y=x^0	; This term only necessary if phases negative
	'P_1': 	begin
		k=1/x	; x must be wavelength[mu]
		w=x^2
;		The refractive index of silica
		ns=sqrt(1+0.6961663*w/(w-0.0684043^2) $
			 +0.4079426*w/(w-0.1162414^2) $
			 +0.8974794*w/(w-9.896161^2))
		y=k*ns
		end
	'P_2': 	begin
		k=1/x
		na=1
		y=k*na
		end
	 else: 	begin
	       	print,'***Error(EVALFUNCTION): unknown function: ',term,'!'
	       	return,-1
	       	end
endcase
return,y
;
end
;-------------------------------------------------------------------------------
function polynom,x,r
;
num=n_elements(x)
degree=n_elements(r)-3
s=r(0:degree)
x_mid=r(degree+1)
x_scl=r(degree+2)
;
terms=['1','x','x^2','x^3','x^4','x^5','x^6']
terms=['L_0','L_1','L_2','L_3','L_4','L_5','L_6']
m=dblarr(num,degree+1)
for i=0,degree do m(*,i)=evalfunction(terms(i),x,x_mid,x_scl)
;
return,m#s
;
end
;-------------------------------------------------------------------------------
function polyfit,x_in,y_in,degree,yft
;
; A better polynomial fit routine than PV-WAVE's own poly_fit.
; Arrays x and y are shifted to improve numerical stability.
;
if degree gt 6 then begin
	print,'***Error(POLYFIT): polynomial order too high!'
	return,0
endif
num=n_elements(x_in)
if num eq 0 then begin
	print,'***Error(POLYFIT): x undefined!'
	return,0
endif
if num ne n_elements(y_in) then begin
	print,'***Error(POLYFIT): number of elements in x and y not equal!'
	return,0
endif
;
; Make sure to remove leading empty dimensions and convert to double
x=double(reform(x_in,num))
y=double(reform(y_in,num))
;
x_mid=(min(x)+max(x))/2
x_scl=2/(max(x)-min(x))
terms=['1','x','x^2','x^3','x^4','x^5','x^6']
terms=['L_0','L_1','L_2','L_3','L_4','L_5','L_6']
m=dblarr(num,degree+1)
for i=0,degree do m(*,i)=evalfunction(terms(i),x,x_mid,x_scl)
tm=transpose(m)
n=tm#m
r=tm#y
s=invert(n)#r
r=[s,[x_mid,x_scl]]
;
yft=polynom(x,r)
return,r
;
end
;-------------------------------------------------------------------------------
function polyres,x_in,y_in,degree
;
r=polyfit(x_in,y_in,degree,yfit)
return,y_in-yfit
;
end
;-------------------------------------------------------------------------------
function perinom,x,r
;
num=n_elements(x)
m=dblarr(num,4,/nozero)
m(*,0)=1
m(*,1)=x
m(*,2)=sin(x)
m(*,3)=cos(x)
return,m#r
;
end
;-------------------------------------------------------------------------------
function perifit,x_in,y_in,period,yft
;
if n_elements(period) eq 0 then begin
	print,'***Error(PERIFIT): period not defined!'
	return,0
endif
num=n_elements(x_in)
if num eq 0 then begin
	print,'***Error(PERIFIT): x undefined!'
	return,0
endif
if num ne n_elements(y_in) then begin
	print,'***Error(PERIFIT): number of elements in x and y not equal!'
	return,0
endif
;
; Make sure to remove leading empty dimensions and convert to double
x=double(reform(x_in,num))/period*2*!pi
y=double(reform(y_in,num))
;
m=dblarr(num,4,/nozero)
m(*,0)=1
m(*,1)=x
m(*,2)=sin(x)
m(*,3)=cos(x)
;
tm=transpose(m)
n=tm#m & m=0
r=tm#y & tm=0
svd8,n,w,u,v
tol=1e-13
small=where(w lt max(w)*tol,count)
print,'SVD: will edit',count,' singular values!'
if count gt 0 then w(small)=0
svb8,u,w,v,r,s
; s=invert(n)#r
;
yft=perinom(x,s)
return,s
;
end
;-------------------------------------------------------------------------------
function perires,x,y,period
;
r=perifit(x,y,period,yfit)
return,y-yfit
;
end
;************************************************************************Block 2
function poweroftwo,series
;
; Reduce to power of 2
;
n=2L^nint(alog10(n_elements(series))/alog10(2))
if n gt n_elements(series) then n=n/2
print,'Number of samples used: ',n
return,series(0:n-1)
;
end
;-------------------------------------------------------------------------------
function powerpatch,t,s,t_new,limit=limit
;
; t must be in ms, type long.
;
if not keyword_set(limit) then limit=100
;
n=n_elements(t)
dt=t(1:n-1)-t(0:n-2)
t_int=long(median(dt))
print,'Sampling interval seems to be ',t_int,' ms'
index=where(dt ne t_int,icount)
print,'Found ',icount,' gaps in time series'
if icount gt 0 then begin
	dtg=t(index+1)-t(index)
	jndex=where(dtg/t_int gt limit,jcount)
	if jcount gt 0 then begin
		print,'Gap(s) too large [s]: ', $
			(t(index(jndex+1))-t(index(jndex)))/1000.0
		kndex=[0-1,index(jndex),n-1]
	endif else kndex=[0-1,n-1]
	nk=n_elements(kndex)
	n1=kndex(0:nk-2)
	n2=kndex(1:nk-1)
	dts=t(n2)-t(n1+1)
	print,'Fixable section(s) [s]: ',dts/1000.0
	i=where(dts eq max(dts)) & i=i(0)
	n1=n1(i)+1
	n2=n2(i)
endif else begin
	n1=0
	n2=n-1
endelse
t_new=t(n1)+(lindgen((t(n2)-t(n1))/t_int+1)*t_int)
return,interpol(s,t,t_new)
;
if count eq 0 then begin
	t_new=t
endif else begin
	n_new=n
	for i=0,count-1 do begin
		dt=t(index(i)+1)-t(index(i))
		num=dt/t_int
		if num lt limit then n_new=n_new+num-1
	endfor
	t_new=lonarr(n_new,/nozero)
	t_new(0:index(0))=t(0:index(0))
	k=index(0)+1
endelse
for i=0,count-1 do begin
	dt=t(index(i)+1)-t(index(i))
	num=dt/t_int
	if num lt limit then begin
		t_new(k:k+num-2)=(lindgen(num-1)+1)*t_int+t(index(i))
		k=k+num-1
	endif
	if i lt count-1 then m=index(i+1) else m=n-1
	t_new(k:k+m-index(i)-1)=t(index(i)+1:m)
	k=k+m-index(i)
;
end
;
n=n_elements(t_new)
dt=t_new(1:n-1)-t_new(0:n-2)
index=where(dt ne t_int,count)
print,'Gaps remaining after patching: ',count,'.'
if count gt 0 then begin
	index=[-1,index,n-1]
	l=index(1:count+1)-index(0:count)
	j=where(l eq max(l))
	t_new=t_new(index(j(0))+1:index(j(0)+1))
endif
;
return,interpol(s,t,t_new)
;
end
;-------------------------------------------------------------------------------
function powerspectrum,series,sampling_int,f,p,f_avg,p_avg
;
; Input series must have power-of-two number of elements.
; Compute power spectrum and return square-root of total
; power, which is equal to the standard deviation of the 
; series.
;
; Compute frequencies and the Fourier transform
;
n=n_elements(series)
m=lindgen(n)
f=m/(n*sampling_int)
p=fft(series,-1)
;
; Frequencies f above the Nyquist frequency are not physically meaningful
; The frequency interval is 1/(n*sampling_int), Nyquist frequency is 
; n/2*frequency_int=1/(2*sampling_int)=1/2 * sampling_rate. In other words,
; the power spectrum is only representative of the series up to the Nyquist
; frequency.
;
n=n/2
;
; We want units in power/Hz. Since there are n frequencies, and the sum
; over n of the power spectrum is normalized, but the frequency range is
; from 0 to (1/sampling_int)/2, the ratio of the two must be multiplied
; with the power.
; 
norm_factor=n/((1/sampling_int)/2)
p=norm_factor*2*abs(p(0:n))^2
f=f(0:n)
;
; Average power in constant logarithmic intervals
num=long(alog10(n)*400)
int=alog10(double(n))/(num-1)
index=nint(10^(dindgen(num)*int))
jndex=uniq(index)
num=n_elements(jndex)-1
f_avg=dblarr(num)
p_avg=f_avg
for i=0,n_elements(jndex)-2 do begin
	f_avg(i)=10^avg(alog10(f(index(jndex(i)):index(jndex(i+1))-1)))
	p_avg(i)=10^avg(alog10(p(index(jndex(i)):index(jndex(i+1))-1)))
endfor
;
; The total power (sum over all elements of p, without norm_factor)
; is equal to the variance of the series. The factor 2 (see above)
; adjusts for the fact that our sum here is only over one side of
; the FFT.
;  
return,sqrt(total(p/norm_factor))
;
;
end
;-------------------------------------------------------------------------------
function siggen,power,wavenumber_int
;
; Wavenumber_int is array [min,max] in 1/microns
;
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
n=n_elements(power)
w=dindgen(n)*(wavenumber_int(1)-wavenumber_int(0))/(n-1)+wavenumber_int(0)
f=c_light*w*1d6
;
p=[power,shift(reverse(power),1)]
s=complex(sqrt(p))
signal=fft(s,1)
;
return,signal
;
end
;-------------------------------------------------------------------------------
function siggen,pwr,fmax
;
; Return signal whose power spectrum is equal to pwr.
; pwr is given for frequencies from zero to fmax [Hz] in equal increments.
;
common LocalSiggen,seed
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
n=n_elements(pwr)-1
sampling_int=1/(2*fmax)
norm_factor=n/((1/sampling_int)/2)
;
p=sqrt(pwr/(2*norm_factor))*exp(2*pi_circle*i_complex*randomu(seed,n+1))
p=[p,conj(reverse(p(1:n-1)))]
s=float(fft(p,1))
return,s
;
end
;-------------------------------------------------------------------------------
pro strucfunc,series,sampling_int,dt,sf,acceleration=factor
;
if not keyword_set(factor) then factor=1.0 else factor=float(factor)
;
n=n_elements(series)
sf=dblarr(n-1)
dt=(dindgen(n-1)+1)*sampling_int
;
num_max=n/factor
;
for i=1l,n-1 do begin
;	series_shift=shift(series,-i)
;	sf(i-1)=avg((series_shift(0:n-1-i)-series(0:n-1-i))^2)
;	sf(i-1)=avg((series(i:n-1)-series(0:n-1-i))^2)
	factor=ceil((n-i)/num_max)
	index=lindgen((n-i)/factor)*factor
	sf(i-1)=avg((series(index+i)-series(index))^2)
endfor
;
!p.multi=0
!p.charsize=1.5
;
plot,dt,sf,/xlog,/ylog, $
	xtitle='Interval [s]'
;
end
;-------------------------------------------------------------------------------
function signalfilter,signal,filter
;
; Filter signal through filter.
;
; Compute power spectrum
n=n_elements(signal)/2
p=fft(signal,-1)
p(0:n-1)=p(0:n-1)*filter
p(n:n*2-1)=p(n:n*2-1)*reverse(filter)
return,float(fft(p,1))
;
end
;-------------------------------------------------------------------------------
function stringl,v,format=form
;
m=1024L
n=n_elements(v)
i=n/m
j=n mod m
;
if n le m then return,string(v,format=form)
;
words=strarr(n)
words(0:m-1)=string(v(0:m-1),format=form)
;
for k=1,i-1 do words(k*m:(k+1)*m-1)=string(v(k*m:(k+1)*m-1),format=form)
if j ne 0 then words(i*m:n-1)=string(v(i*m:n-1),format=form)
;
return,words
;
end
;-------------------------------------------------------------------------------
function myhak,message
;
if n_elements(message) eq 0 then $
	message='<Press Spacebar to continue, q to quit, ? for help>'
print,message,format='(a,$)'
again:
c=get_kbrd(1)
case c of
	'Q':    c='q'
	'H':    c='?'
	'h':    c='?'
	else:
endcase
case c of
'?':    begin
        print,'\x0D',$
              '------------------------------------------         '
        print,'<space>         Display next page of text.
        print,'<return>        Display next line of text.
        print,'q or Q          Quit
        print,'h, H, or ?      Display this message.
        print,'------------------------------------------         '
        goto,again
        end
else:   
endcase
;
return,c
;
end
;-------------------------------------------------------------------------------
function elements,ndim,dims
;
d=lonarr(ndim)+1 & d(ndim-1)=0
n=produkt(dims(0:ndim-1))
v=strarr(n)
for i=0,n-1 do begin
	d(ndim-1)=d(ndim-1)+1
	for j=ndim-1,0,-1 do begin
		if d(j) gt dims(j) then begin
			d(j-1)=d(j-1)+1
			d(j:ndim-1)=1
		endif
	endfor
	s=string(d)+[replicate(',',ndim-1),'']
	sc='' & for j=0,ndim-1 do sc=sc+s(j)
	v(i)=sc
end
;
return,strcompress(v,/remove_all)
;
end
;-------------------------------------------------------------------------------
function printindex,n_total,n_per_line,index_f,index_l
;
common LocalPrintIndex,iline,maxline,extraitems
;
if n_elements(index_l) eq 0 then index_l=-1
;
if index_l lt 0 then begin
	iline=0
	maxline=n_total/n_per_line
	extraitems=n_total mod n_per_line
endif
;
do_print=1
if iline lt maxline then begin
	index_f=iline*n_per_line
	index_l=(iline+1)*n_per_line-1
endif else if iline eq maxline and extraitems gt 0 then begin
	index_f=n_total-extraitems
	index_l=n_total-1
endif else begin
	do_print=0
endelse
iline=iline+1
;
return,do_print
;
end
;-------------------------------------------------------------------------------
function isnumeric,names,strict
;
; Test each element of an array of strings whether it represents a number
; or not. If strict is positive non-zero, do not even allow letters 'e', 'd',
; 'E', and 'D', which can be used in loating point numbers.
;
if n_elements(strict) eq 0 then strict=0 else strict=strict gt 0
;
result=intarr(n_elements(names))
;
for i=0,n_elements(names)-1 do begin
	ascii=byte(strcompress(names(i),/remove_all))
	index=where(ascii lt 43 or ascii gt 57 $
                 or ascii eq 44 or ascii eq 47,count)
	fp=0
	if count eq 0 then begin
		result(i)=1
	endif else if count eq 1 and not strict then begin
		case ascii(index(0)) of
			 68:fp=1
			 69:fp=1
			100:fp=1
			101:fp=1
		       else:fp=0
		endcase
		if fp eq 1 and index(0) ne 0 then result(i)=1
	endif
	jndex=where(ascii eq 43 or ascii eq 45,count)
	if count eq 2 then begin
		if fp eq 0 then result(i)=0 else begin
			if jndex(1) lt index(0) $
			or jndex(0) gt index(0) then result(i)=0
		endelse
	endif else if count gt 2 then result=0	
	index=where(ascii eq 46,count)
	if count gt 1 then result(i)=0
endfor
;
if n_elements(result) eq 1 then return,result(0) else return,result
;
end
;-------------------------------------------------------------------------------
function gitter,num_steps,value_min,value_max
;
; Return an equally spaced array of values centered on zero, i.e.
; beginning at -value_max and ending at +value_max.
;
if n_params() eq 2 then value_max=-value_min
value_step=(value_max-value_min)/(num_steps-1)
return,dindgen(num_steps)*value_step+value_min
;
end
;-------------------------------------------------------------------------------
function mergetable,table1,table2
;
; Merges two tables but does not remove duplicate entries.
;
table=replicate(table1(0),n_elements(table1)+n_elements(table2))
for i=0,n_tags(table)-1 do table.(i)=reform([table1.(i),table2.(i)])
return,table
;
end
;-------------------------------------------------------------------------------
function sizeof,x
;
y=x
;
r=size(y) & n=n_elements(r)
;
if r(n-2) eq 0 then return,0
if r(n-2) ne 8 then y={y:y}
;
tbytes=0L
;
for i=0,n_tags(y)-1 do begin
	r=size(y.(i)) & n=n_elements(r)
	case r(n-2) of
		1:ibytes=1*r(n-1)
		2:ibytes=2*r(n-1)
		3:ibytes=4*r(n-1)
		4:ibytes=4*r(n-1)
		5:ibytes=8*r(n-1)
		6:ibytes=8*r(n-1)
		7:ibytes=1*long(total(strlen(y.(i))))
		8:ibytes=sizeof(y.(i))
		9:ibytes=16*r(n-1)
	       12:ibytes=2*r(n-1)
	       13:ibytes=4*r(n-1)
	       14:ibytes=8*r(n-1)
	       15:ibytes=8*r(n-1)
	endcase
	tbytes=tbytes+ibytes
endfor
;
return,tbytes
;
end
;-------------------------------------------------------------------------------
function natural,x,y
;
return,transpose([[reform(x)],[reform(y)]])
;
end
;-------------------------------------------------------------------------------
function contiguous,index_in,increment
;
if n_elements(increment) eq 0 then increment=1
;
index=index_in
count=n_elements(index)
gaps=index(1:count-1)-index(0:count-2)
gapi=where(gaps gt 1,ngaps)
if ngaps ge 1 then begin
	edge_gap=gapi(0) eq 0
	while edge_gap do begin
		index=index(1:count-1)
		count=count-1
		gaps=index(1:count-1)-index(0:count-2)
		gapi=where(gaps gt 1,ngaps)
		if ngaps eq 0 then edge_gap=0 $
			      else edge_gap=gapi(0) eq 0
	endwhile
	if ngaps gt 0 then begin
	edge_gap=gapi(ngaps-1) eq count-2
	while edge_gap do begin
		index=index(0:count-2)
		count=count-1
		gaps=index(1:count-1)-index(0:count-2)
		gapi=where(gaps gt 1,ngaps)
		if ngaps eq 0 then edge_gap=0 $
			      else edge_gap=gapi(ngaps-1) eq count-2
	endwhile
	endif
	if ngaps gt 0 then begin
		blki=[-1,gapi,count-1]
		nblk=ngaps+1
		blkl=blki(1:nblk)-blki(0:nblk-1)
		iblk=where(blkl eq max(blkl)) & iblk=iblk(0)
		index=index(blki(iblk)+1:blki(iblk)+blkl(iblk))
	endif
endif
return,index
;
end
;-------------------------------------------------------------------------------
function edlen,lambda,p,t,f
;
; Return refractive index for air. lambda[mu], p[mb], t[C], f[mb]
; According to Edlen, ref. K.P. Birch & M.J. Downs, 1993, Metrologia, 30, 155
;
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
if n_elements(lambda) eq 0 then lambda=0.550d0
if n_elements(p) eq 0 then p=1013d0	; mb
if n_elements(t) eq 0 then t=0.0d0	; C
if n_elements(f) eq 0 then f=0.d0	; mb
;
s=1.d0/lambda
a=8343.05d0+2406294.d0/(130.d0-s^2)+15999.d0/(38.9d0-s^2)
b=3.7345d0-0.0401d0*s^2
n=a*p/96095.43d0*(1+1d-8*(0.601d0-0.00972d0*t)*p)/(1+0.003661d0*t)-f*b*1d-2
n=n*1d-6
;
return,n+1
;
end
;-------------------------------------------------------------------------------
function owens,lambda,p,t,f
;
; Return refractive index for air. lambda[mu], p[mb], t[C], f[mb]
; According to Owens, Appl. Opt. 6, 51 (1967)
;
if n_elements(lambda) eq 0 then lambda=0.550d0
if n_elements(p) eq 0 then p=1013d0	; mb
if n_elements(t) eq 0 then t=0.0d0	; C
if n_elements(f) eq 0 then f=0.d0	; mb
;
tk=t+273.15d0	; temperature in Kelvin
;
s=1.d0/lambda
a=(2372.434d0+684255.24d0/(130.d0-s^2)+4549.4d0/(38.9d0-s^2))
b=a-(6487.31d0+58.058d0*s^2-0.7115d0*s^4+0.08851d0*s^6)
n=(a*p-b*f)/tk
n=n*1d-8
;
return,n+1
;
end
;-------------------------------------------------------------------------------
function oneill,lambda,t,r,p
;
; Return refractive index for air according to Owens, with input from the
; NPOI observerlog, i.e. t in Fahrenheit, r relative humidity in %, and
; p in inch Hg. Lambda in microns.
;
; Convert t into Celsius
t0=(t-32)*(5./9)
;
; Convert r into mb at sea level
temp=[-30.,-20,-10,0,10,20,30,40]			; Celsius
evap=[50.88,125.4,286.3,610.8,1227,2337,4243,7378]	; Pascal
e=(r/100)*interpol(evap,temp,t0)/100
;
; Convert e to height 2200 m, altitude of NPOI site
h=2.2	; km
e=e*10^(-h/2)
;
; Convert barometric pressure into mb
p0=33.8*p
;
return,owens(lambda,p0,t0,e)
;
end
;-------------------------------------------------------------------------------
function silica,lambda
;
; Return refractive index of silica.
;
k=1/lambda	; x must be wavelength[mu]
w=lambda^2
ns=sqrt(1+0.6961663*w/(w-0.0684043^2) $
	 +0.4079426*w/(w-0.1162414^2) $
	 +0.8974794*w/(w-9.896161^2))
;
return,ns
;
end
;-------------------------------------------------------------------------------
function extinct,za
;
; Return extinction normalized to extinction at zenith.
; Input zenith angle is in degrees, may be array.
; From: Schlosser, Schmidt-Kaler, Milone, Chall. of Astr., p. 101
;
rad=180/!pi
secza1=1/cos(za/rad)-1
a=0.0018167
b=0.002875
c=0.000808
airmass=secza1+1-a*secza1-b*secza1^2-c*secza1^3
;
; Extinction in the visual is about 0.2 mag per airmass
k=-alog(10^(-0.2/2.5))
return,exp(-k*airmass)/exp(-k)
;
end
;-------------------------------------------------------------------------------
function blackbody,teff,lambda
;
; Return intensity in units of mW m^-2 sr^-1 nm^-1
; (These units are the ones used by Van Hamme in his limb
; darkening tables.)
; teff [K]
; lambda [m]
;
return,(1.191d-22/lambda^5)/(exp(1.439d-2/(lambda*teff))-1)
;
; Old
return,(3.742d-30/lambda^5)/(exp(1.439d-2/(lambda*teff))-1)
;
end
;-------------------------------------------------------------------------------
function jy2n,flux
;
; Return N-band magnitude for a source with flux in Jy.
;
return,4.-2.5*alog(flux)/alog(10)
;
end
;-------------------------------------------------------------------------------
function n2jy,mag
;
; Return flux in Jy for a source of N-band magnitude.
;
return,10^(-(mag-4.)/2.5)
;
end
;-------------------------------------------------------------------------------
function getcolor,i
;
; Return index of a visible tek color. Color indices start at 0 for black.
;
n=1
color=(i mod (32-n-3))+3	; Do not use reserved colors 0-2
;
; Change colors which are problematic
if !d.name eq 'PS' then begin
	if color eq  7 then color=17	; yellow is bad for paper
	if color eq 16 then color=25	; yellow is bad for paper
endif
if color eq 14 then color=31	; dark gray is bad for screen
if color eq  4 then color=28
;
return,color
;
end
;-------------------------------------------------------------------------------
function getsymbol,i
;
; Return a plot symbol other than the invisible dot. i>0.
;
symbols=[1,2,4,5,6,7]
return,symbols((i-1) mod 6)
;
end
;-------------------------------------------------------------------------------
function greek,ch
;
small_g=['alpha','beta','gamma','delta','epsilon','zeta','eta','theta','iota', $
         'kappa','lambda','mu','nu','xi','omi','pi','rho','sigma','tau', $ 
         'ups','phi','chi','psi','omega']
large_g=strupcase(small_g)
all_g=[small_g,large_g]
;
small_a=['a','b','c','d','e','f','g','h','i','j','k','l','m','n', $
	 'o','p','q','r','s','t','u','v','w','x','y','z']
large_a=strupcase(small_a)
all_a=[small_a,large_a]
;
small_l=['a','b','g','d','e','z','h','q','i','k','l','m','n','x','o','p','r', $
	 's','t','u','j','c','y','w']
large_l=strupcase(small_l)
all_l=[small_l,large_l]
;
i=strpos(all_g,ch) & i=where(i eq 0) & i=i(0)
;
if !p.font eq -1 then return,'!4'+all_a(i)+'!X'
if !p.font eq +1 then return,'!M'+all_l(i)+'!X'
;
return,''
;
end
;************************************************************************Block 3
function stringparse,stringg
;
; Simple parser. Example: string='1,3,4-6' --> s=[1,3,4,5,6]
;
for iter=1,2 do begin
	i=0
	pos1=0
	repeat begin
		pos2=strpos(stringg,',',pos1)
		if pos2 eq -1 then pos2=strlen(stringg)
		sub=strmid(stringg,pos1,pos2-pos1)
		index=long(sub)
		if iter eq 2 then s(i)=index
		pos3=strpos(sub,'-',0)
		if pos3 ne -1 then begin
			n=long(strmid(sub,pos3+1,strlen(sub)-pos3-1))
			for k=1,n-index do begin
				i=i+1
				if iter eq 2 then s(i)=index+k
			endfor
		endif
		i=i+1
		pos1=pos2+1
	endrep until pos1 ge strlen(stringg)
	if iter eq 1 then s=intarr(i)	; First run used to set dimension
endfor
return,s
end
;-------------------------------------------------------------------------------
function retroparse,array
;
n=n_elements(array)
if n eq 0 then return,''
index=intarr(n)+1
for i=1,n_elements(array)-2 do begin
	if array(i)-array(i-1) eq 1 and $
	   array(i+1)-array(i) eq 1 then index(i)=0
endfor
stringg=string(array(0))
for i=1,n-1 do begin
	if index(i) then begin
		if index(i-1) then space=',' else space='-'
		stringg=stringg+space+string(array(i))
	endif
endfor
;
return,strcompress(stringg,/remove_all)
end
;-------------------------------------------------------------------------------
function nameparse,name,delimiters,posns
;
; Parse a (single) string into array of words, using delimiters. If at least
; one of the delimiters is a NULL string (i.e. ''), parse into single
; characters, but removing the remaining (single character) delimiters.
; Delimiters default to space and tab characters.
;
if n_elements(delimiters) eq 0 then delimiters=[' ',string(09b)]
;
n=0
j=0
in_word=0
for i=0,strlen(name)-1 do begin
	delimiter=0
	for k=0,n_elements(delimiters)-1 do begin
		if strmid(name,i,1) eq delimiters(k) then begin
			delimiter=1
			if strlen(strtrim(delimiters(k))) eq 0 then isblank=1 $
							       else isblank=0
		endif
	endfor
	if delimiter then begin
		if in_word or not isblank then begin
			n=n+1
			words=strarr(n)
			posns=intarr(n)
			if n gt 1 then words(0:n-2)=old_words
			if n gt 1 then posns(0:n-2)=old_posns
			if in_word then words(n-1)=strmid(name,j,i-j)
			posns(n-1)=j
			old_words=words
			old_posns=posns
			j=i
			in_word=0
		endif
	endif else begin
		if in_word eq 0 then begin
			in_word=1
			j=i
		endif
	endelse
endfor
if in_word eq 1 then begin
	n=n+1
	words=strarr(n)
	posns=intarr(n)
	if n gt 1 then words(0:n-2)=old_words
	if n gt 1 then posns(0:n-2)=old_posns
	words(n-1)=strmid(name,j,i-j)
	posns(n-1)=j
endif
return,words
;
; This is old code, not used anymore
index=where(strlen(delimiters) eq 0,count)
if count gt 0 then begin
	n=0
	index=where(strlen(delimiters) ne 0,count)
	for i=0,strlen(name)-1 do begin
		use=1
		for k=0,count-1 do $
			if strmid(name,i,1) eq delimiters(index(k)) then use=0
		if use then begin
			n=n+1
			words=strarr(n)
			posns=intarr(n)
			if n gt 1 then words(0:n-2)=old_words
			if n gt 1 then posns(0:n-2)=old_posns
			words(n-1)=strmid(name,i,1)
			posns(n-1)=i
			old_words=words
			old_posns=posns
		endif
	endfor
return,words
endif
;
end
;-------------------------------------------------------------------------------
function criparse,request,cat_1,cat_2
;
; Parse cri request into catalog identifiers.
;
len=strlen(request)
pos=strpos(request,'-')
cat_1=strmid(request,0,pos)
cat_2=strmid(request,pos+1,len-pos-1)
if strpos(cat_2,'-') ne -1 then begin
	print,'***Error(CRIPARSE): invalid request!'
	return,-1
endif
return,0
;
end
;-------------------------------------------------------------------------------
function cri,id1,request
end
;-------------------------------------------------------------------------------
function cri,id1,request,lists=lists
;
; Return id2 of star with id1 according to requested crossindex.
;
greek_alphabet=['ALF','ALP','BET','GAM','DEL','EPS','ZET','ETA','THE','TET', $
		'IOT','KAP','LAM', 'MU', 'NU', 'XI','OMI', 'PI','RHO','SIG', $
		'TAU','UPS','PHI','CHI','PSI','OME']
;
if n_params() ne 2 then begin
	print,'***Error(CRI): wrong number of parameters!'
	return,-1
endif
num_id=n_elements(id1)
if num_id eq 0 then begin
	print,'***Error(CRI): id undefined!'
	return,-1
endif
if n_elements(request) eq 0 then begin
	print,'***Error(CRI): request not specified!'
	return,-1
endif else request=strlowcase(request)
;
; If lists=1 search only in non-generic lists
if n_elements(lists) eq 0 then lists=0
;
id2=lonarr(num_id)-1
c2=strarr(num_id)
;
; Special feature: return ID number for name
r=size(id1)
if r(n_elements(r)-2) eq 7 then begin
;	Two-letter catalog names (HD/HR)
	index=where(strupcase(strmid(id1,0,2)) eq 'HD',count)
	if count gt 0 and request eq 'hdn' then id2(index)=long(strmid(id1(index),2,6))
	index=where(strupcase(strmid(id1,0,2)) eq 'HR',count)
	if count gt 0 and request eq 'bsc' then id2(index)=long(strmid(id1(index),2,4))
;
	file=!oyster_dir+'starbase/vlti.bsc'
	result=findfile(file,count=fcount)
	if fcount ne 0 then begin
		bsc=0
		toes=''
		s=dc_read_fixed(file,bsc,toes,/col,ignore=['!'], $
			format='(i4,a32)')
		for i=0,num_id-1 do begin
			words=strupcase(nameparse(id1(i)))
			si=sort(words) & words=words(reverse(si))
			for j=0,n_elements(toes)-1 do begin
				l=0
				name=toes(j)
				for k=0,n_elements(words)-1 do begin
				m=strpos(strupcase(name),words(k),0)
				if m ne -1 then begin
					blank=''
					for n=1,strlen(words(k)) do blank=blank+' '
					strput,name,blank,m
					l=l+1
				endif
				endfor
				if l eq n_elements(words) $
					and strlen(strcompress(toes(j),/remove_all))  $
				 	eq fix(total(strlen(words))) $
			   	then begin
					id2(i)=bsc(j)
					c2(i)='BSC'
				endif
			endfor
		endfor
	endif
	file=!oyster_dir+'starbase/vlti.hic'
	result=findfile(file,count=fcount)
	if fcount ne 0 and total(id2 ge 0) lt num_id then begin
		hic=0
		toes=''
		s=dc_read_fixed(file,hic,toes,/col,ignore=['!'], $
			format='(i6,a32)')
		for i=0,num_id-1 do begin
			words=strupcase(nameparse(id1(i)))
			si=sort(words) & words=words(reverse(si))
			for j=0,n_elements(toes)-1 do begin
				l=0
				name=toes(j)
				for k=0,n_elements(words)-1 do begin
				m=strpos(strupcase(name),words(k),0)
				if m ne -1 then begin
					blank=''
					for n=1,strlen(words(k)) do blank=blank+' '
					strput,name,blank,m
					l=l+1
				endif
				endfor
				if l eq n_elements(words) $
					and strlen(strcompress(toes(j),/remove_all))  $
				 	eq fix(total(strlen(words))) $
			   	then begin
					id2(i)=hic(j)
					c2(i)='HIP'
				endif
			endfor
		endfor
	endif
	file=!oyster_dir+'starbase/toe.bsc'
	result=findfile(file,count=fcount)
	if fcount ne 0 and total(id2 ge 0) lt num_id then begin
		bsc=0
		toes=''
		s=dc_read_fixed(file,bsc,toes,/col,ignore=['!'], $
			format='(i4,a32)')
		for i=0,num_id-1 do begin
			words=strupcase(nameparse(id1(i)))
			si=sort(words) & words=words(reverse(si))
			for j=0,n_elements(toes)-1 do begin
				l=0
				name=toes(j)
				for k=0,n_elements(words)-1 do begin
				m=strpos(strupcase(name),words(k),0)
				if m ne -1 then begin
					blank=''
					for n=1,strlen(words(k)) do blank=blank+' '
					strput,name,blank,m
					l=l+1
				endif
				endfor
				if l eq n_elements(words) $
					and strlen(strcompress(toes(j),/remove_all))  $
				 	eq fix(total(strlen(words))) $
					and id2(i) eq -1 $
			   	then begin
					id2(i)=bsc(j)
					c2(i)='BSC'
				endif
			endfor
		endfor
	endif
	file=!oyster_dir+'starbase/var.bsc'
	result=findfile(file,count=fcount)
	if fcount ne 0 and total(id2 ge 0) lt num_id then begin
		bsc=0
		vars=''
		s=dc_read_free(file,bsc,vars,/col,ignore=['!'])
		for i=0,num_id-1 do begin
			words=strupcase(nameparse(id1(i)))
			si=sort(words) & words=words(reverse(si))
			for j=0,n_elements(vars)-1 do begin
				l=0
				name=vars(j)
				for k=0,n_elements(words)-1 do begin
				m=strpos(strupcase(name),words(k),0)
				if m ne -1 then begin
					blank=''
					for n=1,strlen(words(k)) do blank=blank+' '
					strput,name,blank,m
					l=l+1
				endif
				endfor
				if l eq n_elements(words) $
					and strlen(strcompress(vars(j),/remove_all))  $
				 	eq fix(total(strlen(words))) $
					and id2(i) eq -1 $
			   	then begin
					id2(i)=bsc(j)
					c2(i)='BSC'
				endif
			endfor
		endfor
	endif
	file=!oyster_dir+'starbase/name.bsc'
	result=findfile(file,count=fcount)
	if fcount ne 0 and total(id2 ge 0) lt num_id and not lists then begin
		bsc=0
		names=''
		s=dc_read_free(file,bsc,names,/col,ignore=['!'])
		for i=0,num_id-1 do begin
			words=reverse(strupcase(nameparse(id1(i),[' ','_'])))
			for j=0,n_elements(names)-1 do begin
				l=0
				name=names(j)
				if isnumeric(strmid(name,0,1)) $
					then flamsteed=fix(name) $
					else flamsteed=0
				for k=0,n_elements(words)-1 do begin
				m=strpos(name,strmid(words(k),0,3),/reverse_search)
				if flamsteed ne 0 and isnumeric(words(k)) then $
					if fix(words(k)) ne flamsteed then m=-1
				if k eq 1 and not isnumeric(words(k)) then begin
					index=where(greek_alphabet eq words(k),count)
					if count eq 0 then m=-1
				endif
				if m ne -1 then begin
					l=l+1
					for n=1,strlen(words(k)) do $
						strput,name,' ',m+n-1
				endif
				endfor	
				if l eq n_elements(words) $
					and strlen(names(j)) $
					ge fix(total(strlen(words))) $
					and id2(i) eq -1 $
				then begin
					id2(i)=bsc(j)
					c2(i)='BSC'
				endif
			endfor
		endfor
	endif
	if request eq 'bsc' then begin
		index=where(c2 eq 'HIP',count)
		if count gt 0 then id2(index)=cri(id2(index),'hic-bsc')
		return,id2 
	endif else if request eq 'hic' then begin
		index=where(c2 eq 'BSC',count)
		if count gt 0 then id2(index)=cri(id2(index),'bsc-hic')
		return,id2
	endif else if request eq 'hdn' then begin
		index=where(c2 eq 'BSC',count)
		if count gt 0 then id2(index)=cri(id2(index),'bsc-hdn')
		index=where(c2 eq 'HIP',count)
		if count gt 0 then id2(index)=cri(id2(index),'hic-hdn')
		return,id2
	endif else if request eq 'fkv' then begin
		index=where(c2 eq 'BSC',count)
		if count gt 0 then id2(index)=cri(id2(index),'bsc-fkv')
		index=where(c2 eq 'HIP',count)
		if count gt 0 then id2(index)=cri(id2(index),'hic-fkv')
		return,id2
	endif

endif
		
;
if criparse(request,cat_1,cat_2) ne -1 then begin
        file=!catalogs_dir+'crossindex/'+cat_1+'_'+cat_2+'.cri'
        result=findfile(file,count=fcount)
        if fcount ne 0 then restore,filename=file else begin
                print,'***Error(CRI): file not found: ',file,'!'
                return,-1
        endelse
endif else return,-1
;
num_t=n_elements(t)
if num_id gt num_t/30L then begin
	s=sort(id1)
	id1_s=id1(s)
	id2_s=id2
	j=0L
	for i=0L,num_t-1L do begin
		while t(i).id1 gt id1_s(j) and j lt num_id-1L do j=j+1L
		while t(i).id1 eq id1_s(j) and j lt num_id-1L do begin
			id2_s(j)=t(i).id2
			j=j+1L
		endwhile
		if t(i).id1 eq id1_s(j) then id2_s(j)=t(i).id2
	endfor
	id2(s)=id2_s
endif else begin
	for i=0L,num_id-1L do begin
		index=where(t.id1 eq id1(i),count)
		if count ne 0 then id2(i)=t(index(0)).id2
	endfor
endelse
;
if n_elements(id2) eq 1 then return,id2(0) else return,id2
;
end
;-------------------------------------------------------------------------------
function hms,x,aspro=aspro
;
; Return formatted string for x in hours. x may be array.
;
if n_elements(aspro) eq 0 then aspro=0
;
index=where(x lt 0,count)
sign=strarr(n_elements(x)) & sign(*)='+'
if count gt 0 then sign(index)='-'
sign(*)=''
x=abs(x)
hour=fix(x)
min=fix((x-hour)*60)
sec=float(((x-hour)*60-min)*60)
if aspro then begin
sign='   +'
label= sign+strcompress(string(hour,format='(i2.2)'))+':' $
      +strcompress(string(min,format='(i2.2)'))+':' $
      +strcompress(string(fix(sec),format='(i2.2)'))+'.' $
      +strcompress(string((sec-fix(sec))*1000,format='(i3.3)'))
endif else begin
label= strcompress(sign+string(hour,format='(i2.2)'))+'h ' $
      +strcompress(string(min,format='(i2.2)'))+'m ' $
      +strcompress(string(sec,format='(f4.1)'))+'s'
endelse
if n_elements(label) eq 1 then return,label(0) else return,label
;
end
;-------------------------------------------------------------------------------
function dms,x,aspro=aspro
;
; Return formatted string for x in degrees. x may be array.
;
if n_elements(aspro) eq 0 then aspro=0
;
index=where(x lt 0,count)
sign=strarr(n_elements(x)) & sign(*)='+'
if count gt 0 then sign(index)='-'
x=abs(x)
deg=fix(x)
min=fix((x-deg)*60)
sec=float(((x-deg)*60-min)*60)
if aspro then begin
if index(0) ge 0 then deg(index)=-deg(index)
label= '  '+string(deg,format='(i4.2)')+':' $
      +strcompress(string(min,format='(i2.2)'))+":" $
      +strcompress(string(fix(sec),format='(i2.2)'))+"." $
      +strcompress(string((sec-fix(sec))*1000,format='(i3.3)'))
endif else begin
label= strcompress(sign+string(deg,format='(i3.2)'))+'d ' $
      +strcompress(string(min,format='(i2.2)'))+"' " $
      +strcompress(string(sec,format='(f6.3)'))+'"'
endelse
if n_elements(label) eq 1 then return,label(0) else return,label
;
end
;-------------------------------------------------------------------------------
function hms2h,h
;
; Return floating point time in hours from hh:mm:ss.sss input. Arrays OK.
;
t=double(strmid(h,0,2))
t=t+double(strmid(h,3,2))/60
t=t+double(strmid(h,6,strlen(h)-6))/3600
return,t
;
end
;-------------------------------------------------------------------------------
function dms2d,d
;
; Return floating point angle in degrees from dd:mm:ss.sss input. Arrays OK.
;
a=double(strmid(d,0,2))
a=a+double(strmid(d,3,2))/60
a=a+double(strmid(d,6,strlen(d)-6))/3600
return,a
;
end
;-------------------------------------------------------------------------------
function wdsid,ra,dec
;
; Return positions in WDS ID format string, i.e. +HHMMFDDMMF,
; or -HHMMFDDMMF, with the sign corresponding to the sign of the
; declination.
;
n=n_elements(ra)
;
rah=fix(ra)
ram=(ra-rah)*60
raf=(ram-fix(ram))*10
decd=fix(dec)
decm=(abs(dec-decd))*60
decf=(decm-fix(decm))*10
index=where(decd lt 0,count)
wds=strarr(n)+'+' & if count gt 0 then wds(index)='-'
wds=wds+string(rah,format='(i2.2)') $
       +string(fix(ram),format='(i2.2)') $
       +string(fix(raf),format='(i1.1)') $
       +string(abs(decd),format='(i2.2)') $
       +string(fix(decm),format='(i2.2)') $
       +string(fix(decf),format='(i1.1)')
;
return,wds
;
end
;-------------------------------------------------------------------------------
function esoid,ra,dec
;
; Return positions in ESO ID format string, i.e. +HHMMSSFFDDMMSSF,
; or -HHMMSSFFDDMMSSF, with the sign corresponding to the sign of the
; declination.
;
n=n_elements(ra)
;
rah=fix(ra)
ram=fix((ra-rah)*60)
ras=(ra-rah-ram/60.)*3600
raf=(ras-fix(ras))*100
decd=fix(dec)
decm=fix((abs(dec-decd))*60)
decs=(abs(dec)-abs(decd)-decm/60.)*3600
decf=(decs-fix(decs))*10
index=where(decd lt 0,count)
wds=strarr(n)+'+' & if count gt 0 then wds(index)='-'
wds=wds+string(rah,format='(i2.2)') $
       +string(fix(ram),format='(i2.2)') $
       +string(fix(ras),format='(i2.2)') $
       +string(fix(raf),format='(i2.2)') $
       +string(abs(decd),format='(i2.2)') $
       +string(fix(decm),format='(i2.2)') $
       +string(fix(decs),format='(i2.2)') $
       +string(fix(decf),format='(i1.1)')
;
return,wds
;
end
;-------------------------------------------------------------------------------
function wdspos,id
;
; Given a WDS ID format string, convert to RA and DEC.
;
rah=float(strmid(id,1,2))
ram=float(strmid(id,3,3))/10
decd=float(strmid(id,6,2))
decm=float(strmid(id,8,3))/10
sign=strmid(id,0,1)
f=fltarr(n_elements(id))+1
index=where(sign eq '-',count)
if count gt 0 then f(index)=-1
;
return,[[rah+ram/60],[f*(decd+decm/60)]]
;
end
;-------------------------------------------------------------------------------
function esopos,id
;
; Given an ESO ID format string, convert to RA and DEC.
;
rah=float(strmid(id,1,2))
ram=float(strmid(id,3,2))
ras=float(strmid(id,5,4))/100
decd=float(strmid(id,9,2))
decm=float(strmid(id,11,2))
decs=float(strmid(id,13,3))/10
sign=strmid(id,0,1)
f=fltarr(n_elements(id))+1
index=where(sign eq '-',count)
if count gt 0 then f(index)=-1
;
return,[[rah+ram/60+ras/3600],[f*(decd+decm/60+decs/3600)]]
;
end
;-------------------------------------------------------------------------------
function npoifile,filename
;
; Extract date portion if one of the following standard NPOI files:
; YYYY-MM-DD.extension[.NNN.NN], with the following extensions below.
; (Raw HDS files have an additional scan.sub-scan extension.)
;
extensions=['.raw','.con','.coh','.cha','.inch']
i=0
repeat begin
	file=strmid(filename,0,strpos(filename,extensions(i)))
	i=i+1
endrep until strlen(file) ne 0 or i eq n_elements(extensions)
if strlen(file) ne 0 then begin
	while strpos(file,'/') ne -1 do $
		file=strmid(file,strpos(file,'/')+1,strlen(file))
endif else begin
	print,'Warning(NPOIFILE): non-standard extension!'
	if strpos(filename,'/') lt 0 then file=strmid(filename,0,10) $
				     else file='YYYY-MM-DD'
endelse
;
return,file
;
end
;-------------------------------------------------------------------------------
function pathname,filespec
;
; Return path in an absolute file name. Includes the last '/'.
;
path=''
if n_elements(filespec) eq 0 then filespec=''
name=filespec
while strpos(name,'/') ne -1 do begin
        path=path+strmid(name,0,strpos(name,'/')+1)
        name=strmid(name,strpos(name,'/')+1,strlen(name))
endwhile
;
if strlen(filespec) eq 0 then begin
	spawn,'pwd',path & path=path(0)
	path=path+'/'
endif
;
return,path
;
end
;-------------------------------------------------------------------------------
function specname,filespec
;
; Return name in file path.
;
n=n_elements(filespec)
if n eq 0 then return,''
names=strarr(n)
for i=0,n-1 do begin
	if strlen(filespec(i)) ne 0 then $
	names(i)=strmid(filespec(i),strlen(pathname(filespec(i))),strlen(filespec(i)))
endfor
;
if n eq 1 then return,names(0) else return,names
;
end
;-------------------------------------------------------------------------------
function finddir,path
;
if n_elements(path) eq 0 then path=''
;
spawn,'unalias ls;which ls',command
spawn,command+' -aF '+path,result
spawn,command+' -a  '+path,dirs
index=where(strpos(result,'/') ne -1,count)
if count gt 0 then return,dirs(index) else return,''
;
end
;-------------------------------------------------------------------------------
function finddir,path
;
if n_elements(path) eq 0 then path='.'
default_dirs=['.','..']
;
files=findfile(path+'/*')
index=where(strpos(files,':') ne -1,count)
if count eq 0 then return,default_dirs
dirs=specname(files(index))
for i=0,count-1 do dirs(i)=strmid(dirs(i),0,strlen(dirs(i))-1)
return,[default_dirs,dirs]
;
end
;-------------------------------------------------------------------------------
function safe,command
;
; Make sure now spawn command is used with 'rm *' anywhere in it.
;
if strpos(command,'rm ') ne -1 then begin
	if strpos(command,'*') ne -1 then return,0
endif
;
return,1
;
end
;-------------------------------------------------------------------------------
function breve,b
;
; Return the reversed baseline.
;
return,strmid(b,4,3)+'-'+strmid(b,0,3)
;
end
;************************************************************************Block 4
function set_boxes,box_x,box_y,nbox,clear=clear
;
; Lets user place boxes in a plot for various purposes.
; Returns button status to identify user command, as well as
; the coordinates, box_x and box_y, and the number, nbox, of the boxes.
;
if n_elements(clear) eq 0 then clear=0 else clear=clear gt 0
if clear then begin
	if n_elements(nbox) eq 0 then nbox=1
	device,set_graphics_function=6
	for i=0,nbox-1 do plots,box_x(*,i),box_y(*,i),psym=0
	device,set_graphics_function=3
	return,0
endif
;
if n_params() eq 2 then begin
	boxmode=1
	numbox=1
	print,'______________________________________________________'
	print,'Place one box by clicking left mouse button to anchor '
	print,'a corner of the box. Click middle to fix size and to  '
	print,'move box around. Click left button to set the box.'
	print,'Click right button to cancel/reset and return!'
	print,'________________________***___________________________'
endif else begin
	boxmode=0
	numbox=20
	print,'_________________________________________________________'
	print,'You have a maximum of',numbox,' boxes to place! To begin:'
	print,'click left button to anchor a corner of the box. Click   '
	print,'middle button to fix size and to move box around. Click  '
	print,'left button to fix the box.'
	print,'Click right mouse button to exit to edit menu.'
	print,'_________________________***______________________________'
endelse
;
; Box corners start at top left corner and continue clockwise
box_x=fltarr(5,numbox)
box_y=fltarr(5,numbox)
nbox=-1
ierr=0
while((nbox lt numbox-1) and (ierr ne 4))do begin
cursor,x,y,/down & ierr=!err
if ierr eq 1 then begin
	nbox=nbox+1
	box_x(*,nbox)=x
	box_y(*,nbox)=y
;	Use 10 or 6 here
	device,set_graphics_function=6
	while((ierr ne 2) and (ierr ne 4)) do begin
		cursor,x,y,/change & ierr=!err
		plots,box_x(*,nbox),box_y(*,nbox),psym=0
		box_x(1:2,nbox)=x & box_y(2:3,nbox)=y
		plots,box_x(*,nbox),box_y(*,nbox),psym=0
	endwhile
	device,set_graphics_function=3
endif
if ierr eq 2 then begin
	if nbox ge 0 then begin
		device,set_graphics_function=6
		while((ierr ne 1) and (ierr ne 4)) do begin
			x0=x & y0=y
			cursor,x,y,/change & ierr=!err
			plots,box_x(*,nbox),box_y(*,nbox),psym=0
			box_x(*,nbox)=box_x(*,nbox)+x-x0 
			box_y(*,nbox)=box_y(*,nbox)+y-y0
			plots,box_x(*,nbox),box_y(*,nbox),psym=0
		endwhile
		device,set_graphics_function=3
	endif
endif
if nbox ge 0 then begin
	if box_x(1,nbox) lt box_x(0,nbox) then begin
		box_x(*,nbox)=shift(box_x(*,nbox),-2)
		box_x(3,nbox)=box_x(4,nbox)
	endif
	if box_y(2,nbox) gt box_y(1,nbox) then begin
		box_y(*,nbox)=shift(box_y(*,nbox),-2)
		box_y(4,nbox)=box_y(0,nbox)
	endif
endif
endwhile
icom=ierr
nbox=nbox+1
return,icom
;
if boxmode eq 0 then begin
	icom=cw_form('0,LIST,Delete inside boxes|Delete outside boxes|Start over again|Cancel and return,set_value=0,QUIT',title='Question?')
	icom=icom.tag0+1
	if icom.tag0 eq 3 then begin
		device,set_graphics_function=6
		for i=0,nbox do plots,box_x(*,i),box_y(*,i),psym=0
		device,set_graphics_function=3
;   		goto,boxes
	endif
endif else icom=ierr
if icom eq 4 then begin
	device,set_graphics_function=6
	for i=0,nbox do plots,box_x(*,i),box_y(*,i),psym=0
	device,set_graphics_function=3
endif
return,icom
;
end
;-------------------------------------------------------------------------------
function set_points,plotdata_x,plotdata_y,index
;
print,'___________________________'
print,'Click left button on point.'
print,'Click right button to exit!'
print,'___________***_____________'
cursor,x,y,/down
icom=!err
dist_x=(plotdata_x-x)/(!x.crange(1)-!x.crange(0))*!d.x_size
dist_y=(plotdata_y-y)/(!y.crange(1)-!y.crange(0))*!d.y_size
dist=(dist_x^2+dist_y^2)
index=where(dist eq min(dist)) & index=index(0)
tvcrs,plotdata_x(index),plotdata_y(index),/data
return,icom
;
end
;-------------------------------------------------------------------------------
function get_ellipse,p,a1,a2,e_parms,mode,phase
;
; e_parms=fltarr(5)
; 0: center x-coordinate
; 1: center y-coordinate
; 2: major axis
; 3: minor axis
; 4: position angle
;
c=[(a1(0)+a2(0))/2,(a1(1)+a2(1))/2]
ca1=a1-c
asqr=total(ca1^2)
if mode eq 1 then begin
        cp=c-p
        cpsqr=total(cp^2)
        xsqr=total(ca1*cp)^2/asqr
        ysqr=cpsqr-xsqr
        bsqr=ysqr/(1-xsqr/asqr)
        b=sqrt(bsqr)
endif else b=e_parms(3)
;
pa=atan(ca1(0),ca1(1))
r=ellipse(sqrt(asqr),b,phase-pa)
x=r*sin(phase)+c(0)
y=r*cos(phase)+c(1)
;
e_parms(0:1)=c
e_parms(2)=sqrt(asqr)
e_parms(3)=b
e_parms(4)=pa
;
return,[[x],[y]]
end
;-------------------------------------------------------------------------------
function set_ellipse
;
common Constants,c_light,pi_circle,e_euler,i_complex,a_disp,b_disp
;
print,'________________________________________________________'
print,'Use mouse buttons to define apparent ellipse:'
print,'Click left  button to  anchor major axis, right to fix.'
print,'Click middle button to adjust minor axis, right to fix.'
print,'Click right button to exit (major axis will dissappear).'
print,'__________________________***___________________________'
;
e_parms=dblarr(5)
;
ax=dblarr(2)
ay=dblarr(2)
;
n=90
phase=dindgen(n+1)*2*pi_circle/n
;
status=''
err1=0
device,set_graphics_function=6
!p.psym=0
;
WHILE ERR1 NE 4 DO BEGIN
;
!err=0
while !err ne 1 and !err ne 2 and !err ne 4 do cursor,x,y,/down
err1=!err
case status of
''  :	begin
     	case err1 of
     	1:	begin
  		status='a'
		ax(*)=x
		ay(*)=y
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		end
	2:	print,'Please anchor a first!'
	else:
	endcase
	end
'aa':	begin
  	case err1 of
	1:	begin
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		dist_x=(ax-x)/(!x.crange(1)-!x.crange(0))*!d.x_size
		dist_y=(ay-y)/(!y.crange(1)-!y.crange(0))*!d.y_size
		dist=(dist_x^2+dist_y^2)
		s=sort(dist)
		s=s([1,0])
		ax=ax(s)
		ay=ay(s)
		ax(1)=x
		ay(1)=y
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		tvcrs,ax(1),ay(1),/data
		end
	2:	begin
		status='ab'
		v=get_ellipse([x,y],[ax(0),ay(0)],[ax(1),ay(1)],e_parms,1,phase)
		plots,v(*,0),v(*,1)
		end
	else:
	endcase	
	end
'ab':	begin
	case err1 of
	1:	begin
		status='el'
		plots,v(*,0),v(*,1)
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
                dist_x=(ax-x)/(!x.crange(1)-!x.crange(0))*!d.x_size
                dist_y=(ay-y)/(!y.crange(1)-!y.crange(0))*!d.y_size
                dist=(dist_x^2+dist_y^2)
                s=sort(dist)
                s=s([1,0])
                ax=ax(s)
                ay=ay(s)
                ax(1)=x
                ay(1)=y
                plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
                tvcrs,ax(1),ay(1),/data
		v=get_ellipse([x,y],[ax(0),ay(0)],[ax(1),ay(1)],e_parms,0,phase)
		plots,v(*,0),v(*,1)
		end
	2:	begin
		plots,v(*,0),v(*,1)
		v=get_ellipse([x,y],[ax(0),ay(0)],[ax(1),ay(1)],e_parms,1,phase)
		plots,v(*,0),v(*,1)
		end
	else:
	endcase
	end
else:
endcase
if status ne '' and err1 ne 4 then begin
err2=0
while err2 ne 4 do begin
	cursor,x,y,/change & err2=!err
	case status of
	'a':	begin
		new_status='aa'
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		ax(1)=x
		ay(1)=y
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		end
	'aa':	begin
		new_status='aa'
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		ax(1)=x
		ay(1)=y
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		end
	'ab':	begin
		new_status='ab'
		plots,v(*,0),v(*,1)
		v=get_ellipse([x,y],[ax(0),ay(0)],[ax(1),ay(1)],e_parms,1,phase)
		plots,v(*,0),v(*,1)
		end
	'el':	begin
		new_status='ab'
		plots,v(*,0),v(*,1)
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		ax(1)=x
		ay(1)=y
		plots,ax,ay
		plots,total(ax)/2,total(ay)/2,psym=4
		v=get_ellipse([x,y],[ax(0),ay(0)],[ax(1),ay(1)],e_parms,0,phase)
		plots,v(*,0),v(*,1)
		end
	endcase
endwhile
status=new_status
endif
!err=err1
;
ENDWHILE
;
plots,ax,ay
device,set_graphics_function=3
return,e_parms
;
end
;-------------------------------------------------------------------------------
